#include "combopage.h"

ComboBoxPage::ComboBoxPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  itemGrp = new QGroupBox(this);
  itemGrp->setGeometry(0, 0, 310, 110);
  itemGrp->setTitle("Items");
  itemEdit = new QLineEdit(itemGrp);
  itemEdit->setGeometry(5, 20, 145, 25);
  addBut = new QPushButton(itemGrp);
  addBut->setGeometry(5, 50, 70, 25);
  addBut->setText("Add");
  insBut = new QPushButton(itemGrp);
  insBut->setGeometry(80, 50, 70, 25);
  insBut->setText("Insert");
  chgBut = new QPushButton(itemGrp);
  chgBut->setGeometry(5, 80, 70, 25);
  chgBut->setText("Change");
  delBut = new QPushButton(itemGrp);
  delBut->setGeometry(80, 80, 70, 25);
  delBut->setText("Delete");
  itemList = new QListBox(itemGrp);
  itemList->setGeometry(155, 20, 150, 85);
  optGrp = new QGroupBox(this);
  optGrp->setGeometry(0, 110, 310, 75);
  optGrp->setTitle("Options");
  untitled_0 = new QLabel(optGrp);
  untitled_0->setGeometry(5, 15, 85, 25);
  untitled_0->setText("Maximum lines");
  lineEdit = new QLineEdit(optGrp);
  lineEdit->setGeometry(90, 15, 100, 25);
  aresBox = new QCheckBox(optGrp);
  aresBox->setGeometry(90, 45, 100, 25);
  aresBox->setText("Auto Resize");

  connect(addBut, SIGNAL(clicked()), SLOT(addEntry()) );
  connect(addBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(insBut, SIGNAL(clicked()), SLOT(insEntry()) );
  connect(insBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(chgBut, SIGNAL(clicked()), SLOT(chgEntry()) );
  connect(chgBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(delBut, SIGNAL(clicked()), SLOT(delEntry()) );
  connect(delBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(itemList, SIGNAL(highlighted(int)), SLOT(newItem(int)) );
  connect(itemEdit, SIGNAL(returnPressed()), SLOT(addEntry()) );
}

ComboBoxPage::~ComboBoxPage()
{
}

void ComboBoxPage::setTarget(QWidget *target)
{
  _target = (QComboBox *)target;
  itemEdit->setText("");
  itemList->clear();
  for(int i=0; i<_target->count(); i++)
    itemList->insertItem(_target->text(i));
  aresBox->setChecked(_target->autoResize());
  QString buffer;
  buffer.setNum(_target->sizeLimit());
  lineEdit->setText(buffer);
  dirty = 0;
}

void ComboBoxPage::applyValues()
{
  if(! dirty)
    return;
  _target->clear();
  for(int i=0; i<(int)itemList->count(); i++)
    _target->insertItem(itemList->text(i));
  _target->setAutoResize(aresBox->isChecked());
  _target->setSizeLimit(atoi(lineEdit->text()));
}

void ComboBoxPage::addEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  itemList->insertItem(itemEdit->text());
}

void ComboBoxPage::insEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  if(itemList->currentItem() >= 0)
    itemList->insertItem(itemEdit->text(), itemList->currentItem());
  else
    itemList->insertItem(itemEdit->text());
}

void ComboBoxPage::chgEntry()
{
 if(*itemEdit->text() == '\0')
   return;
 if(itemList->currentItem() >= 0)
   itemList->changeItem(itemEdit->text(), itemList->currentItem());
}

void ComboBoxPage::delEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  if(itemList->currentItem() >= 0)
    itemList->removeItem(itemList->currentItem());
}

void ComboBoxPage::newItem(int idx)
{
  itemEdit->setText(itemList->text(idx));
}
