#include <qobjcoll.h>
#include <qmsgbox.h>
#include <qfiledlg.h>
#include <qobjcoll.h>
#include <qfile.h>

#include "pixmaps/buttongroup.xpm"
#include "pixmaps/checkbox.xpm"
#include "pixmaps/combobox.xpm"
#include "pixmaps/dialog.xpm"
#include "pixmaps/frame.xpm"
#include "pixmaps/groupbox.xpm"
#include "pixmaps/ico_unkn.xpm"
#include "pixmaps/label.xpm"
#include "pixmaps/lineedit.xpm"
#include "pixmaps/listbox.xpm"
#include "pixmaps/menubar.xpm"
#include "pixmaps/multilineedit.xpm"
#include "pixmaps/pushbutton.xpm"
#include "pixmaps/radiobutton.xpm"
#include "pixmaps/scrollbar.xpm"
#include "pixmaps/slider.xpm"
#include "pixmaps/widget.xpm"

#include "dsgnmdl.h"
#include "winfomodal.h"

//
// External globals for other modules
//
extern StorageModule *store;
extern PrefModule *prefs;

/*!
  \fn void DesignModule::quit()

  This signal is emitted by the DesignModule when file->exit was selected
  from the menu bar.
*/

/*!
  \fn DesignModule::~DesignModule()
  
  Delete a DesignModule
*/

/*!
  Create a design module. This will also create a number of other modules
  like MemberFuncListModule, MemberVarListModule and InclListModule.
*/

DesignModule::DesignModule(QWidget *parent, const char *name)
: QWidget(parent, name)
{
  setCaption("Tree View");
  resize(300, 400);
  place = 0;
  wInfo = 0;
  nextNumber = 0;

  valDialog = new ValModal(this, "");
  valDialog->hide();

  mfWin = new MemberFuncListModule(0, "");
  mfWin->hide();
  mvWin = new MemberVarListModule(0, "");
  mvWin->hide();
  incWin = new InclListModule(0, "");
  incWin->hide();

  backPopup = new QPopupMenu(0);
  backPopup->insertItem("&New..", this, SLOT(newApp()) );
  backPopup->insertItem("&Save..", this, SLOT(save()) );
  backPopup->insertItem("&Load..", this, SLOT(load()) );
  backPopup->insertSeparator();
  backPopup->insertItem("&Export", this, SLOT(export()) );
  backPopup->insertSeparator();
  backPopup->insertItem("&Preferences", this, SLOT(editPrefs()) );
  backPopup->insertSeparator();
  backPopup->insertItem("E&xit", this, SLOT(stopApp()) );

  addPopup = new QPopupMenu(0);
  addPopup->insertItem("CheckBox");
  addPopup->insertItem("PushButton");
  addPopup->insertItem("RadioButton");
  addPopup->insertItem("ComboBox");
  addPopup->insertItem("Frame");
  addPopup->insertItem("GroupBox");
  addPopup->insertItem("ButtonGroup");
  addPopup->insertItem("Label");
  addPopup->insertItem("MenuBar");
  addPopup->insertItem("ListBox");
  addPopup->insertItem("MultiLineEdit");
  addPopup->insertItem("LineEdit");
  addPopup->insertItem("ScrollBar");
  addPopup->insertItem("Slider");
  connect(addPopup, SIGNAL(activated(int)), SLOT(addNode(int)) );

  nodePopup = new QPopupMenu(0);
  nodePopup->insertItem("&Delete", this, SLOT(deleteNode()) );
  nodePopup->insertItem("&Add", addPopup);
  nodePopup->insertItem("&Rename", this, SLOT(renameNode()) );
  nodePopup->insertSeparator();
  nodePopup->insertItem("&Properties", this, SLOT(editNode()) );

  editaddPopup = new QPopupMenu(0);
  editaddPopup->insertItem("CheckBox");
  editaddPopup->insertItem("PushButton");
  editaddPopup->insertItem("RadioButton");
  editaddPopup->insertItem("ComboBox");
  editaddPopup->insertItem("Frame");
  editaddPopup->insertItem("GroupBox");
  editaddPopup->insertItem("ButtonGroup");
  editaddPopup->insertItem("Label");
  editaddPopup->insertItem("MenuBar");
  editaddPopup->insertItem("ListBox");
  editaddPopup->insertItem("MultiLineEdit");
  editaddPopup->insertItem("LineEdit");
  editaddPopup->insertItem("ScrollBar");
  editaddPopup->insertItem("Slider");
  connect(editaddPopup, SIGNAL(activated(int)), SLOT(addNode(int)) );

  editPopup = new QPopupMenu(0);
  editPopup->insertItem("&Delete", this, SLOT(deleteNode()) );
  editPopup->insertItem("&Rename", this, SLOT(renameNode()) );
  editPopup->insertItem("&Inheritance", this, SLOT(editInh()) );
  editPopup->insertItem("&Add", editaddPopup);
  editPopup->insertSeparator();
  editPopup->insertItem("&Properties", this, SLOT(editNode()) );

  windowPopup = new QPopupMenu(0);
  windowPopup->setCheckable(TRUE);
  windowPopup->insertItem("Member Functions");
  windowPopup->insertItem("Member Variables");
  windowPopup->insertItem("Include Files");
  windowPopup->insertItem("Design");
  windowPopup->setItemChecked(3, TRUE);
  connect(windowPopup, SIGNAL(activated(int)), SLOT(changeVisibility(int)) );

  menu = new QMenuBar(this);
  menu->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  menu->insertItem("&File", backPopup);
  menu->insertItem("&Edit", editPopup);
  menu->insertItem("&Window", windowPopup);
  menu->setItemEnabled(1, FALSE);
  menu->setItemEnabled(2, FALSE);
  menu->setItemEnabled(3, FALSE);
  menu->update();

  tree = new TreeWidget(this);
  tree->setGeometry(0, menu->height(), width(), height()-menu->height());
  tree->setNodePopup(nodePopup);
  tree->setBackPopup(backPopup);

  pics[0] = new QPixmap((const char **)checkbox_xpm);
  pics[1] = new QPixmap((const char **)pushbutton_xpm);
  pics[2] = new QPixmap((const char **)radiobutton_xpm);
  pics[3] = new QPixmap((const char **)combobox_xpm);
  pics[4] = new QPixmap((const char **)frame_xpm);
  pics[5] = new QPixmap((const char **)groupbox_xpm);
  pics[6] = new QPixmap((const char **)buttongroup_xpm);
  pics[7] = new QPixmap((const char **)label_xpm);
  pics[8] = new QPixmap((const char **)menubar_xpm);
  pics[9] = new QPixmap((const char **)listbox_xpm);
  pics[10] = new QPixmap((const char **)multilineedit_xpm);
  pics[11] = new QPixmap((const char **)lineedit_xpm);
  pics[12] = new QPixmap((const char **)scrollbar_xpm);
  pics[13] = new QPixmap((const char **)slider_xpm);
  pics[14] = new QPixmap((const char **)widget_xpm);
  pics[15] = new QPixmap((const char **)dialog_xpm);
  pics[16] = new QPixmap((const char **)ico_unkn);

  connect(tree, SIGNAL(highlighted()), SLOT(actNode()) );
  connect(tree, SIGNAL(selected()), SLOT(editNode()) );
  connect(tree, SIGNAL(nothingSelected()), SLOT(stopSelection()) );
}

/*!
  Create a new design. This slot is called when file->new is selected.
*/

void DesignModule::newApp()
{
  if(wInfo)
  {
    int ret = QMessageBox::warning(this, "Ebuilder",
                       "Are you sure?\nThis will overwrite your current design"
                       " with the new design?", "&Yes", "&No", 0, 0, 1);
    if(ret)
      return;
  }

  WidgetInfoModal wmodal(this, "");
  wmodal.exec();
  if(wmodal.result())
  {
    WidgetInfo *info = wmodal.getInfo();
    if(!checkName(info->name))
    {
      delete info;
      return;
    }
    if(wInfo)
      delete wInfo;

    if(info->baseSubstitute == "QWidget")
      info->widget = new QWidget(0, info->name);
    else //if(info->baseSubstitute == "QDialog")
      info->widget = new QDialog(0, info->name, info->modal);

    info->widget->show();
    info->widget->setStyle(prefs->defaultStyle());
    info->widget->resize(400, 300);
    info->widget->setSizeIncrement(prefs->xGrid(), prefs->yGrid());
    tree->clear();
    tree->addNode(info->name, getPixmap(info->widget), info->widget);
    place = new PlaceHolder(info->widget);
    place->setTarget(0);
    place->setGrid(prefs->xGrid(), prefs->yGrid());

    // Add two functions to the member function list
    MemberFunction *ctor = new MemberFunction;
    ctor->type = "";
    ctor->name = info->name;
    ctor->arglist = info->arguments;
    ctor->access = "public";
    ctor->extra = "ctor";
    ctor->text = "";
    info->funcList.append(ctor);
    MemberFunction *dtor = new MemberFunction;
    dtor->type = "";
    dtor->name = "~";
    dtor->name = dtor->name + info->name;
    dtor->arglist = "";
    dtor->access = "public";
    dtor->extra = "dtor";
    dtor->text = "";
    info->funcList.append(dtor);

    // Set the list properties of the member function and include windows
    mfWin->setList(info->funcList);
    mvWin->setList(info->varList);
    incWin->setList(info->incList);

    // Enable menu items
    menu->setItemEnabled(1, TRUE);
    menu->setItemEnabled(2, TRUE);
    menu->setItemEnabled(3, TRUE);
    menu->update();

    wInfo = info;
  }
}

/*!
  Save the current design to disk using a proprietary format. This slot is
  activated when file->save is selected.
*/

void DesignModule::save()
{
  if(!wInfo)
  {
    QMessageBox::information(this, "Ebuilder",
                         "Nothing to save. Please design something "
                         "first (try file->new)");
    return;
  }
  QString sname(tree->rootNode()->text());
  sname += ".wdg";
  QString fname = QFileDialog::getSaveFileName(0, "*.wdg", this, sname);
  if(fname.isNull())
    return;
  if(!fname.isEmpty())
  {
    if(QFile::exists(fname))
    {
      int ret = QMessageBox::warning(this, "Ebuilder"
                                     , "Overwrite existing file?"
                                     , "&Yes", "&No", 0, 0, 1);
      if(ret)
        return;
    }

    if(wInfo)
      store->write(wInfo, fname);
    QMessageBox::information(this, "Ebuilder", "Save operation complete");
  }
}

/*!
  Load a design from disk. The design should be store in the proprietary
  format used by Ebuilder. If any error occurs during loading, the current
  design will not be overwritten.
*/

void DesignModule::load()
{
  if(wInfo)
  {
    int ret = QMessageBox::warning(this, "Ebuilder",
                       "Are you sure?\nThis will overwrite your current design"
                       " with the new design?", "&Yes", "&No", 0, 0, 1);
    if(ret)
      return;
  }
  QString fname = QFileDialog::getOpenFileName(0, "*.wdg", this);
  if(fname.isNull())
    return;
  if(!fname.isEmpty())
  {
    WidgetInfo *info;
    info = store->read(fname);

    if(!info)
    {
      QMessageBox::critical(this, "Ebuilder", "Error reading widget file");
      return;
    }
    if(wInfo)
      delete wInfo;
    wInfo = info;
    wInfo->widget->setSizeIncrement(prefs->xGrid(), prefs->yGrid());

    // Move the design to a sane location
    wInfo->widget->move(x()+width()+20, y());
  }
  else
    return;

  // Reconstruct the node tree
  tree->clear();
  rebuildTree(wInfo->widget, 0);

  // Create a new placeholder
  place = new PlaceHolder(wInfo->widget);
  place->setTarget(0);
  place->setGrid(prefs->xGrid(), prefs->yGrid());
  nextNumber += 50;

  // Set the list properties of the member function and include windows
  mfWin->setList(wInfo->funcList);
  mvWin->setList(wInfo->varList);
  incWin->setList(wInfo->incList);

  // Enable menu items
  menu->setItemEnabled(1, TRUE);
  menu->setItemEnabled(2, TRUE);
  menu->setItemEnabled(3, TRUE);
  menu->update();
}

/*!
  Export the current design to C++. This slot is activated when file->export
  is selected.
*/

void DesignModule::export()
{
  if(wInfo)
  {
    valDialog->setQuestion("Filename base for .cc and .h files:");
    valDialog->setText(tree->rootNode()->text());
    valDialog->showAtCursor();
    if(valDialog->result())
    {
      store->export(wInfo, valDialog->text());
      QMessageBox::information(this, "Ebuilder", "Export complete");
    }
  }
  else
    QMessageBox::information(this, "Ebuilder",
                         "Nothing to export. Please design something "
                         "first! (try file->new)");
}

/*!
  Emits a quit signal.
*/

void DesignModule::stopApp()
{
  emit quit();
}

/*!
  Add a subwidget to the currently selected widget and let the user set its
  properties.
*/

void DesignModule::addNode(int type)
{
  if(tree->currentNode()  == 0)
  {
    QMessageBox::information(this, "Ebuilder",
                         "Please select the node you wish to add\n"
                         "to with the left mouse button before\n"
                         "using this function");
    return;
  }

  valDialog->setQuestion("Enter name of new object:");
  QString buffer;
  buffer.setNum(nextNumber++);
  buffer = "untitled_" + buffer;
  valDialog->setText(buffer);
  valDialog->showAtCursor();
  if(!valDialog->result())
    return;
  if(! checkName(valDialog->text()))
    return;

  QWidget *newWidget;
  QWidget *parent = (QWidget *)(tree->currentNode()->data());
  QString pclass = parent->className();
  if(pclass == "QMultiLineEdit" || pclass == "QListBox")
  {
    QMessageBox::information(this, "Ebuilder",
                         "Adding subwidgets to QMultiLineEdit or\n"
                         "QListBox is not supported");
    return;
  }
  QPixmap *picture=0;

  if(type == 0)
  {
    newWidget = new QCheckBox(parent, valDialog->text());
    picture = pics[0];
  }
  else if(type == 1)
  {
    newWidget = new QPushButton(parent, valDialog->text());
    picture = pics[1];
  }
  else if(type == 2)
  {
    newWidget = new QRadioButton(parent, valDialog->text());
    picture = pics[2];
  }
  else if(type == 3)
  {
    newWidget = new QComboBox(FALSE, parent, valDialog->text());
    picture = pics[3];
  }
  else if(type == 4)
  {
    newWidget = new QFrame(parent, valDialog->text());
    picture = pics[4];
  }
  else if(type == 5)
  {
    newWidget = new QGroupBox(parent, valDialog->text());
    picture = pics[5];
  }
  else if(type == 6)
  {
    newWidget = new QButtonGroup(parent, valDialog->text());
    picture = pics[6];
  }
  else if(type == 7)
  {
    newWidget = new QLabel(parent, valDialog->text());
    picture = pics[7];
  }
  else if(type == 8)
  {
    newWidget = new QMenuBar(parent, valDialog->text());
    picture = pics[8];
  }
  else if(type == 9)
  {
    newWidget = new QListBox(parent, valDialog->text());
    picture = pics[9];
  }
  else if(type == 10)
  {
    newWidget = new QMultiLineEdit(parent, valDialog->text());
    picture = pics[10];
  }
  else if(type == 11)
  {
    newWidget = new QLineEdit(parent, valDialog->text());
    picture = pics[11];
  }
  else if(type == 12)
  {
    newWidget = new QScrollBar(parent, valDialog->text());
    picture = pics[12];
  }
  else //(type == 13)
  {
    newWidget = new QSlider(parent, valDialog->text());
    picture = pics[13];
  }

  newWidget->setStyle(prefs->defaultStyle());
  newWidget->show();
  WidgetInspector tmp(this);
  tmp.setTarget(newWidget);
  tmp.exec();

  tree->addNode(valDialog->text(), picture, newWidget, tree->currentNode());
}

/*!
  Delete the currently selected node and all its children from the design.
*/

void DesignModule::deleteNode()
{
  if(tree->currentNode() == 0)
  {
    QMessageBox::information(this, "Ebuilder",
                         "Please select the node you wish to delete\n"
                         "with the left mouse button before using\n"
                         "this function");
    return;
  }
  int ret = QMessageBox::warning(this, "Ebuilder", "Are you sure you want\n"
	      "to delete the selected node?", "&Yes", "&No", 0, 0, 1);
  if(ret)
    return;

  QWidget *target = (QWidget *)(tree->currentNode()->data());
  place->setTarget(0);
  place->hide();
  tree->rmNode(tree->currentNode());
  delete target;
}

/*!
  Rename the current node and its accompanying widget. If the current node
  is the root node, the WidgetInfo data will be updated as well.
*/

void DesignModule::renameNode()
{
  if(tree->currentNode() == 0)
  {
    QMessageBox::information(this, "Ebuilder",
                         "Please select the node you wish to rename\n"
                         "with the left mouse button before using\n"
                         "this function");
    return;
  }
  valDialog->setQuestion("Enter new name of node:");
  QWidget *target = (QWidget *)(tree->currentNode()->data());
  valDialog->setText(target->name());
  valDialog->showAtCursor();
  if(valDialog->result())
  {
    target->setName(valDialog->text());
    tree->currentNode()->setText(valDialog->text());
    tree->update();
    if(tree->currentNode() == tree->rootNode())
    {
      wInfo->name = valDialog->text();
      QListIterator<MemberFunction> memfIt(wInfo->funcList);
      for(; memfIt.current(); ++memfIt)
      {
        MemberFunction *curr = memfIt.current();
        if(curr->extra == "ctor")
          curr->name = wInfo->name;
        else if(curr->extra == "dtor")
          curr->name = "~" + wInfo->name;
      }
      mfWin->setList(wInfo->funcList);
    }
  }
}

/*!
  Edit the properties of the currently selected node using a WidgetInspector.
*/

void DesignModule::editNode()
{
  if(tree->currentNode() == 0)
  {
    QMessageBox::information(this, "Ebuilder",
                         "Please select the node you wish to edit\n"
                         "with the left mouse button before using\n"
                         "this function");
    return;
  }
  QWidget *target = (QWidget *)(tree->currentNode()->data());
  WidgetInspector tmp(this);
  tmp.setTarget(target);
  tmp.exec();
}

/*!
  Activate a node after it has been selected. This moves the PlaceHolder to
  the widget associated with this node.
*/

void DesignModule::actNode()
{
  QWidget *target = (QWidget *)(tree->currentNode()->data());
  if(target->parent() == 0)
    return;
  if(place)
    place->setTarget(target);
}

/*!
  Unselects all nodes and hides the PlaceHolder.
*/

void DesignModule::stopSelection()
{
  if(place)
  {
    place->setTarget(0);
    place->hide();
  }
}

/*!
  Update the visibility of the different windows such as the member
  function window.
*/

void DesignModule::changeVisibility(int idx)
{
  if(! wInfo)
  {
    QMessageBox::information(this, "Ebuilder",
                         "Please design something first (try file->new)");
    return;
  }

  if(windowPopup->isItemChecked(idx))
  {
    windowPopup->setItemChecked(idx, FALSE);
    if(idx == 0)
      mfWin->hide();
    else if(idx == 1)
      mvWin->hide();
    else if(idx == 2)
      incWin->hide();
    else //(idx == 3)
      wInfo->widget->hide();
  }
  else
  {
    windowPopup->setItemChecked(idx, TRUE);
    if(idx == 0)
      mfWin->show();
    else if(idx == 1)
      mvWin->show();
    else if(idx == 2)
      incWin->show();
    else //(idx == 3)
      wInfo->widget->show();
  }
}

/*!
  Edit the preferences for Ebuilder. These include x- and y grid settings
  and extensions used for exporting C++ code.
*/

void DesignModule::editPrefs()
{
  prefs->exec();
  if(prefs->result())
  {
    if(wInfo)
      wInfo->widget->setSizeIncrement(prefs->xGrid(), prefs->yGrid());
    if(place)
      place->setGrid(prefs->xGrid(), prefs->yGrid());
  }
}

/*!
  Edit the inheritance properties of the design. This may force us to
  re-load the whole design. This happens if the substitute class is changed
  from QWidget to QDialog or vice-versa.
*/

void DesignModule::editInh()
{
  if(! wInfo)
  {
    QMessageBox::information(this, "Ebuilder",
                             "Please create a design with file->new first");
    return;
  }
  WidgetInfoModal wmodal(this, "");
  wmodal.setInfo(wInfo);
  wmodal.exec();
  if(wmodal.result())
  {
    WidgetInfo *info = wmodal.getInfo();
    if(!checkName(info->name))
    {
      delete info;
      return;
    }
    if(info->name != wInfo->name)
    {
      wInfo->name = info->name;
      tree->rootNode()->setText(info->name);
      tree->update();

      QListIterator<MemberFunction> memfIt(wInfo->funcList);
      for(; memfIt.current(); ++memfIt)
      {
        MemberFunction *curr = memfIt.current();
        if(curr->extra == "ctor")
          curr->name = info->name;
        else if(curr->extra == "dtor")
          curr->name = "~" + info->name;
      }
      mfWin->setList(wInfo->funcList);
    }
    if(info->arguments != wInfo->arguments)
    {
      wInfo->arguments = info->arguments;
      // change ctor
      QListIterator<MemberFunction> memfIt(wInfo->funcList);
      for(; memfIt.current(); ++memfIt)
      {
        MemberFunction *curr = memfIt.current();
        if(curr->extra == "ctor")
        {
          curr->arglist = info->arguments;
          break;
        }
      }
    }
    if(info->baseClass != wInfo->baseClass)
      wInfo->baseClass = info->baseClass;
    if(info->baseInitializer != wInfo->baseInitializer)
      wInfo->baseInitializer = info->baseInitializer;
    if(info->baseHeader != wInfo->baseHeader)
      wInfo->baseHeader = info->baseHeader;
    if(info->modal != wInfo->modal)
      wInfo->modal = info->modal;
    if(info->baseSubstitute != wInfo->baseSubstitute)
    {
      wInfo->baseSubstitute = info->baseSubstitute;
      // save and re-load whole widget
      QMessageBox::information(this, "Ebuilder",
                               "You changed the base class substitute.\n"
                               "The widget needs to be saved and re-loaded\n"
                               "to complete this operation.\n"
                               "The widget will be save to '__tmp.wdg'" );
      store->write(wInfo, "__tmp.wdg");

      // re-load
      WidgetInfo *wtmp = store->read("__tmp.wdg");
      if(! wtmp)
      {
        QMessageBox::critical(this, "Ebuilder", "Error reading widget");
        return;
      }
      delete wInfo;
      wInfo = wtmp;

      wInfo->widget->setSizeIncrement(prefs->xGrid(), prefs->yGrid());
      tree->clear();
      rebuildTree(wInfo->widget, 0);
      place = new PlaceHolder(wInfo->widget);
      place->setTarget(0);
      place->setGrid(prefs->xGrid(), prefs->yGrid());
      mfWin->setList(wInfo->funcList);
      mvWin->setList(wInfo->varList);
      incWin->setList(wInfo->incList);
    }

    delete info;
  }
}

//
// Event handlers
//

/*!
  Handle resize events. This mainly resizes the TreeWidget since the menu
  bar resizes itself.
*/

void DesignModule::resizeEvent(QResizeEvent *)
{
  tree->setGeometry(0, menu->height(), width(), height()-menu->height());
}

/*!
  Check names for validity. Empty names or names with special characters in
  them are not accepted.
*/

bool DesignModule::checkName(const char *name)
{
  // Check for an empty string
  if(*name == '\0')
  {
    QMessageBox::critical(this, "Ebuilder", "You must name a widget");
    return FALSE;
  }

  // Check for illegal characters in the name
  QString badchar("~`!@#$%^&*()-+=|\\[{]};:'\",<.>/? ");
  QString str(name);

  int bclen = badchar.length();
  for(int i=0; i<bclen; i++)
  {
    if(str.find(badchar[i]) >= 0)
    {
      QMessageBox::critical(this, "Ebuilder", "Invalid name");
      return FALSE;
    }
  }

  // Check for names starting with a number
  QString numchar = "0123456789";
  for(unsigned int j=0; j<numchar.length(); j++)
  {
    if(str[0] == numchar[j])
      return FALSE;
  }

  return TRUE;
}

/*!
  Rebuild the tree view of a widget. For each subwidget a TreeNode is
  created and added to the TreeWidget. This member function is recursive.
*/

void DesignModule::rebuildTree(QWidget *target, TreeNode *par)
{
  TreeNode *currNode;

  // Create a new tree node;
  currNode = tree->addNode(target->name(), getPixmap(target), target, par);

  // Ask for all children in the first degree of this widget
  QString className = target->className();
  if(className != "QListBox" && className != "QMultiLineEdit")
  {
    QObjectList *list = target->queryList("QWidget", 0, FALSE, FALSE);
    QObjectListIt it(*list);
    while(it.current())
    {
      rebuildTree((QWidget *)it.current(), currNode);
      ++it;
    }
    delete list;
  }
}

/*!
  Return the correct QPixmap to use for a widget type in the TreeWidget.
*/

QPixmap *DesignModule::getPixmap(QWidget *target)
{
  QString type(target->className());

  if(type == "QCheckBox")
    return pics[0];
  else if(type == "QPushButton")
    return pics[1];
  else if(type == "QRadioButton")
    return pics[2];
  else if(type == "QComboBox")
    return pics[3];
  else if(type == "QFrame")
    return pics[4];
  else if(type == "QGroupBox")
    return pics[5];
  else if(type == "QButtonGroup")
    return pics[6];
  else if(type == "QLabel")
    return pics[7];
  else if(type == "QMenuBar")
    return pics[8];
  else if(type == "QListBox")
    return pics[9];
  else if(type == "QMultiLineEdit")
    return pics[10];
  else if(type == "QLineEdit")
    return pics[11];
  else if(type == "QScrollBar")
    return pics[12];
  else if(type == "QSlider")
    return pics[13];
  else if(type == "QWidget")
    return pics[14];
  else if(type == "QDialog")
    return pics[15];
  else
    return pics[16];
}
