#ifndef DesignModule_included
#define DesignModule_included

#include <qwidget.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qlist.h>

#include "shared.h"
#include "tree.h"
#include "inspect.h"
#include "placehld.h"
#include "storemdl.h"
#include "valmodal.h"
#include "mflmdl.h"
#include "mvlmdl.h"
#include "incmdl.h"
#include "prefmdl.h"

/*!
  \class DesignModule dsgnmdl.h
  \brief The main design window of Ebuilder holding the TreeWidget

  The DesignModule is the main window of Ebuilder. All operations like load,
  save, export and construction of new subwidgets is done through this
  window.
*/

class DesignModule: public QWidget
{
  Q_OBJECT
  public:
    DesignModule(QWidget *parent=0, const char *name=0);
    ~DesignModule() {delete[] pics;}
  private slots:
    void newApp();
    void save();
    void load();
    void export();
    void stopApp();
    void addNode(int);
    void deleteNode();
    void editNode();
    void renameNode();
    void actNode();
    void stopSelection();
    void changeVisibility(int);
    void editPrefs();
    void editInh();
  signals:
    void quit();
  private:
    void resizeEvent(QResizeEvent *);
    bool checkName(const char *name);
    void rebuildTree(QWidget *target, TreeNode *parent);
    QPixmap *getPixmap(QWidget *target);
  private:
    //! The popup menu used on the TreeWidget nodes
    QPopupMenu *nodePopup;
    //! The popup menu used for adding new nodes
    QPopupMenu *addPopup;
    //! The popup used for the TreeWidget background
    QPopupMenu *backPopup;
    //! The popup used for the \e edit menu item
    QPopupMenu *editPopup;
    QPopupMenu *editaddPopup;
    //! The popup used for the \e window menu item
    QPopupMenu *windowPopup;
    //! The menu bar
    QMenuBar *menu;

    //! The TreeWidget used for the hierarchical widget view
    TreeWidget *tree;
    //! The PlaceHolder used for widget resize/move operations
    PlaceHolder *place;
    //! A general modal for asking questions to the user
    ValModal *valDialog;
    //! The member function window
    MemberFuncListModule *mfWin;
    //! The member variable window
    MemberVarListModule *mvWin;
    //! The include file window
    InclListModule *incWin;

    //! All info about the widget that's being designed
    WidgetInfo *wInfo;
    //! The pixmaps used for the different widget types
    QPixmap *pics[17];

    //! The next free number for automatic widget naming
    int nextNumber;

    //! The header extension used for exporting
    QString hExt;
    //! The implementation extension used for exporting
    QString iExt;
    //! The directory used for exporting
    QString expDir;
    //! The size of the layout grid in the x-direction
    int gridx;
    //! The size of the layout grid in the y-direction
    int gridy;
};
    
#endif
