#include "./fontpage.h"

FontPage::FontPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(330, 160);
  famGrp = new QButtonGroup(this);
  famGrp->setGeometry(0, 0, 170, 80);
  famGrp->setTitle("Family");
  famGrp->setExclusive((bool)1);
  fstdBut = new QRadioButton(famGrp);
  fstdBut->setGeometry(5, 15, 20, 25);
  fstdBut->setChecked((bool)1);
  fudefBut = new QRadioButton(famGrp);
  fudefBut->setGeometry(5, 45, 20, 25);
  fontCombo = new QComboBox(famGrp);
  fontCombo->setGeometry(20, 15, 120, 25);
  fontCombo->insertItem("Helvetica");
  fontCombo->insertItem("Times");
  fontCombo->insertItem("Courier");
  fontCombo->insertItem("OldEnglish");
  fontCombo->insertItem("System");
  fontEdit = new QLineEdit(famGrp);
  fontEdit->setGeometry(20, 45, 120, 25);
  styleGrp = new QGroupBox(this);
  styleGrp->setGeometry(0, 80, 170, 80);
  styleGrp->setTitle("Style");
  itBox = new QCheckBox(styleGrp);
  itBox->setGeometry(5, 15, 80, 25);
  itBox->setText("Italic");
  strBox = new QCheckBox(styleGrp);
  strBox->setGeometry(85, 15, 80, 25);
  strBox->setText("Strikeout");
  ulBox = new QCheckBox(styleGrp);
  ulBox->setGeometry(5, 45, 80, 25);
  ulBox->setText("Underline");
  fixBox = new QCheckBox(styleGrp);
  fixBox->setGeometry(85, 45, 80, 25);
  fixBox->setText("Fixed");
  weightGrp = new QButtonGroup(this);
  weightGrp->setGeometry(175, 0, 155, 80);
  weightGrp->setTitle("Weight");
  weightGrp->setExclusive((bool)1);
  weightCombo = new QComboBox(weightGrp);
  weightCombo->setGeometry(25, 15, 95, 25);
  weightCombo->insertItem("Light");
  weightCombo->insertItem("Normal");
  weightCombo->insertItem("DemiBold");
  weightCombo->insertItem("Bold");
  weightCombo->insertItem("Black");
  wstdBut = new QRadioButton(weightGrp);
  wstdBut->setGeometry(5, 15, 20, 25);
  wstdBut->setChecked((bool)1);
  wudefBut = new QRadioButton(weightGrp);
  wudefBut->setGeometry(5, 45, 20, 25);
  weightEdit = new QLineEdit(weightGrp);
  weightEdit->setGeometry(25, 45, 95, 25);
  sizeGrp = new QButtonGroup(this);
  sizeGrp->setGeometry(175, 80, 155, 80);
  sizeGrp->setTitle("Point Size");
  sizeGrp->setExclusive((bool)1);
  sstdBut = new QRadioButton(sizeGrp);
  sstdBut->setGeometry(5, 15, 20, 25);
  sstdBut->setChecked((bool)1);
  sudefBut = new QRadioButton(sizeGrp);
  sudefBut->setGeometry(5, 45, 20, 25);
  sizeCombo = new QComboBox(sizeGrp);
  sizeCombo->setGeometry(25, 15, 50, 25);
  sizeCombo->insertItem("8");
  sizeCombo->insertItem("10");
  sizeCombo->insertItem("12");
  sizeCombo->insertItem("14");
  sizeCombo->insertItem("18");
  sizeCombo->insertItem("24");
  sizeCombo->insertItem("36");
  sizeCombo->insertItem("48");
  sizeCombo->insertItem("72");
  sizeCombo->insertItem("96");
  sizeEdit = new QLineEdit(sizeGrp);
  sizeEdit->setGeometry(25, 45, 50, 25);

  connect(famGrp, SIGNAL(clicked(int)), SLOT(toggleFont(int)) );
  connect(sizeGrp, SIGNAL(clicked(int)), SLOT(toggleSize(int)) );
  connect(weightGrp, SIGNAL(clicked(int)), SLOT(toggleWeight(int)) );
  connect(fontEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(weightEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(sizeEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(fontCombo, SIGNAL(highlighted(int)), SLOT(setDirty(int)) );
  connect(weightCombo, SIGNAL(highlighted(int)), SLOT(setDirty(int)) );
  connect(sizeCombo, SIGNAL(highlighted(int)), SLOT(setDirty(int)) );
}

FontPage::~FontPage()
{
}

void FontPage::setTarget(QWidget *target)
{
  _target = target;
  QFont fnt = _target->font();
  fontCombo->setEnabled(TRUE);
  fontEdit->setEnabled(FALSE);
  fstdBut->setChecked(TRUE);
  fudefBut->setChecked(FALSE);
  QString fam = fnt.family();
  if(fam == "Helvetica")
    fontCombo->setCurrentItem(0);
  else if(fam == "Times")
    fontCombo->setCurrentItem(1);
  else if(fam == "Courier")
    fontCombo->setCurrentItem(2);
  else if(fam == "OldEnglish")
    fontCombo->setCurrentItem(3);
  else if(fam == "System")
    fontCombo->setCurrentItem(4);
  else
  {
    fontCombo->setEnabled(FALSE);
    fontEdit->setEnabled(TRUE);
    fstdBut->setChecked(FALSE);
    fudefBut->setChecked(TRUE);
    fontEdit->setText(fam);
  }
  
  sizeCombo->setEnabled(TRUE);
  sizeEdit->setEnabled(FALSE);
  sstdBut->setChecked(TRUE);
  sudefBut->setChecked(FALSE);
  int fsize = fnt.pointSize();
  if(fsize == 8)
    sizeCombo->setCurrentItem(0);
  else if(fsize == 10)
    sizeCombo->setCurrentItem(1);
  else if(fsize == 12)
    sizeCombo->setCurrentItem(2);
  else if(fsize == 14)
    sizeCombo->setCurrentItem(3);
  else if(fsize == 18)
    sizeCombo->setCurrentItem(4);
  else if(fsize == 24)
    sizeCombo->setCurrentItem(5);
  else if(fsize == 36)
    sizeCombo->setCurrentItem(6);
  else if(fsize == 48)
    sizeCombo->setCurrentItem(7);
  else if(fsize == 72)
    sizeCombo->setCurrentItem(8);
  else if(fsize == 96)
    sizeCombo->setCurrentItem(9);
  else
  {
    sizeCombo->setEnabled(FALSE);
    sizeEdit->setEnabled(TRUE);
    sstdBut->setChecked(FALSE);
    sudefBut->setChecked(TRUE);
    QString buffer;
    buffer.setNum(fsize);
    sizeEdit->setText(buffer);
  }
  
  weightCombo->setEnabled(TRUE);
  weightEdit->setEnabled(FALSE);
  wstdBut->setChecked(TRUE);
  wudefBut->setChecked(FALSE);
  int wgt = fnt.weight();
  if(wgt == QFont::Light)
    weightCombo->setCurrentItem(0);
  else if(wgt == QFont::Normal)
    weightCombo->setCurrentItem(1);
  else if(wgt == QFont::DemiBold)
    weightCombo->setCurrentItem(2);
  else if(wgt == QFont::Bold)
    weightCombo->setCurrentItem(3);
  else if(wgt == QFont::Black)
    weightCombo->setCurrentItem(4);
  else
  {
    weightCombo->setEnabled(FALSE);
    weightEdit->setEnabled(TRUE);
    wstdBut->setChecked(FALSE);
    wudefBut->setChecked(TRUE);
    QString buffer;
    buffer.setNum(wgt);
    weightEdit->setText(buffer);
  }
  
  itBox->setChecked(fnt.italic());
  ulBox->setChecked(fnt.underline());
  strBox->setChecked(fnt.strikeOut());
  fixBox->setChecked(fnt.fixedPitch());
  
  dirty = 0;
}

void FontPage::applyValues()
{
  if(dirty == 0)
    return;
  
  const char *family;
  int psize = 12, weight = QFont::Normal;
  
  if(fstdBut->isChecked())
  {
    family = fontCombo->currentText();
  }
  else
  {
    if(*fontEdit->text() != '\0')
      family = fontEdit->text();
    else
      family = "Times";
  }
  
  if(wstdBut->isChecked())
  {
    switch(weightCombo->currentItem())
    {
      case 0:
        weight = QFont::Light;
        break;
      case 1:
        weight = QFont::Normal;
        break;
      case 2:
        weight = QFont::DemiBold;
        break;
      case 3:
        weight = QFont::Bold;
        break;
      default:
        weight = QFont::Black;
    }
  }
  else
  {
    if(*weightEdit->text() != '\0')
      weight = atoi(weightEdit->text());
  }
  
  if(sstdBut->isChecked())
  {
    psize = atoi(sizeCombo->currentText());
  }
  else
  {
    if(*sizeEdit->text() != '\0')
      psize = atoi(sizeEdit->text());
  }
  
  QFont fnt(family, psize, weight, itBox->isChecked());
  fnt.setUnderline(ulBox->isChecked());
  fnt.setStrikeOut(strBox->isChecked());
  fnt.setFixedPitch(fixBox->isChecked());
  
  _target->setFont(fnt);
}

void FontPage::toggleFont(int but)
{
  dirty = 1;
  if(but)
  {
    fontCombo->setEnabled(FALSE);
    fontEdit->setEnabled(TRUE);
  }
  else
  {
    fontCombo->setEnabled(TRUE);
    fontEdit->setEnabled(FALSE);
  }
}

void FontPage::toggleSize(int but)
{
  dirty = 1;
  if(but)
  {
    sizeCombo->setEnabled(FALSE);
    sizeEdit->setEnabled(TRUE);
  }
  else
  {
    sizeCombo->setEnabled(TRUE);
    sizeEdit->setEnabled(FALSE);
  }
}

void FontPage::toggleWeight(int but)
{
  dirty = 1;
  if(but)
  {
    weightCombo->setEnabled(FALSE);
    weightEdit->setEnabled(TRUE);
  }
  else
  {
    weightCombo->setEnabled(TRUE);
    weightEdit->setEnabled(FALSE);
  }
}
