#include "framepage.h"

FramePage::FramePage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  styleGrp = new QGroupBox(this);
  styleGrp->setGeometry(0, 0, 310, 55);
  styleGrp->setTitle("Frame Style");
  shapeCombo = new QComboBox(styleGrp);
  shapeCombo->setGeometry(10, 20, 80, 25);
  shapeCombo->insertItem("None");
  shapeCombo->insertItem("Box");
  shapeCombo->insertItem("Panel");
  shapeCombo->insertItem("WinPanel");
  shapeCombo->insertItem("HLine");
  shapeCombo->insertItem("VLine");
  shadowCombo = new QComboBox(styleGrp);
  shadowCombo->setGeometry(95, 20, 80, 25);
  shadowCombo->insertItem("None");
  shadowCombo->insertItem("Plain");
  shadowCombo->insertItem("Raised");
  shadowCombo->insertItem("Sunken");
  lwGrp = new QGroupBox(this);
  lwGrp->setGeometry(0, 55, 150, 55);
  lwGrp->setTitle("Line Width");
  lwEdit = new QLineEdit(lwGrp);
  lwEdit->setGeometry(10, 20, 80, 25);
  mlwGrp = new QGroupBox(this);
  mlwGrp->setGeometry(155, 55, 155, 55);
  mlwGrp->setTitle("Midline Width");
  mlwEdit = new QLineEdit(mlwGrp);
  mlwEdit->setGeometry(10, 20, 80, 25);

  connect(shapeCombo, SIGNAL(highlighted(int)), SLOT(setDirty(int)) );
  connect(shadowCombo, SIGNAL(highlighted(int)), SLOT(setDirty(int)) );
  connect(lwEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(mlwEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
}

FramePage::~FramePage()
{
}

void FramePage::setTarget(QWidget *target)
{
  _target = (QFrame *)target;
  shapeCombo->setCurrentItem(_target->frameShape());
  shadowCombo->setCurrentItem(_target->frameShadow() / 16);
  QString buffer;
  buffer.setNum(_target->lineWidth());
  lwEdit->setText(buffer);
  buffer.setNum(_target->midLineWidth());
  mlwEdit->setText(buffer);
  dirty = 0;
}

void FramePage::applyValues()
{
  if(! dirty)
    return;
  _target->setFrameStyle(shapeCombo->currentItem() +
                         shadowCombo->currentItem() * 16);
  _target->setLineWidth(atoi(lwEdit->text()));
  _target->setMidLineWidth(atoi(mlwEdit->text()));
}
