#include "geopage.h"

GeoPage::GeoPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(320, 180);
  posGrp = new QGroupBox(this);
  posGrp->setGeometry(0, 0, 320, 50);
  posGrp->setTitle("Position (x, y)");
  xEdit = new QLineEdit(posGrp);
  xEdit->setGeometry(5, 15, 100, 25);
  yEdit = new QLineEdit(posGrp);
  yEdit->setGeometry(110, 15, 100, 25);
  sizeGrp = new QGroupBox(this);
  sizeGrp->setGeometry(0, 50, 320, 50);
  sizeGrp->setTitle("Size (width, height)");
  wEdit = new QLineEdit(sizeGrp);
  wEdit->setGeometry(5, 15, 100, 25);
  hEdit = new QLineEdit(sizeGrp);
  hEdit->setGeometry(110, 15, 100, 25);
  fixBut = new QPushButton(sizeGrp);
  fixBut->setGeometry(215, 15, 100, 25);
  fixBut->setText("Fix Size");
  extrGrp = new QGroupBox(this);
  extrGrp->setGeometry(0, 100, 320, 80);
  extrGrp->setTitle("Extremes (width, height)");
  minwEdit = new QLineEdit(extrGrp);
  minwEdit->setGeometry(5, 15, 100, 25);
  minhEdit = new QLineEdit(extrGrp);
  minhEdit->setGeometry(110, 15, 100, 25);
  maxwEdit = new QLineEdit(extrGrp);
  maxwEdit->setGeometry(5, 45, 100, 25);
  maxhEdit = new QLineEdit(extrGrp);
  maxhEdit->setGeometry(110, 45, 100, 25);
  untitled_0 = new QLabel(extrGrp);
  untitled_0->setGeometry(215, 15, 100, 25);
  untitled_0->setText("(Minimum)");
  untitled_1 = new QLabel(extrGrp);
  untitled_1->setGeometry(215, 45, 100, 25);
  untitled_1->setText("(Maximum)");

  connect(fixBut, SIGNAL(clicked()), SLOT(fixsize()) );
  connect(fixBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(xEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(yEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(wEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(hEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(minwEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(minhEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(maxwEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
  connect(maxhEdit, SIGNAL(textChanged(const char *)), SLOT(setDirty(const char *)) );
}

GeoPage::~GeoPage()
{
}

void GeoPage::setTarget(QWidget *target)
{
  _target = target;
  QString buffer;
  buffer.setNum(_target->x());
  xEdit->setText(buffer);
  buffer.setNum(_target->y());
  yEdit->setText(buffer);
  buffer.setNum(_target->width());
  wEdit->setText(buffer);
  buffer.setNum(_target->height());
  hEdit->setText(buffer);
  buffer.setNum(_target->minimumSize().width());
  minwEdit->setText(buffer);
  buffer.setNum(_target->minimumSize().height());
  minhEdit->setText(buffer);
  buffer.setNum(_target->maximumSize().width());
  maxwEdit->setText(buffer);
  buffer.setNum(_target->maximumSize().height());
  maxhEdit->setText(buffer);
  dirty = 0;
}

void GeoPage::applyValues()
{
  if(! dirty)
    return;
  _target->setGeometry(atoi(xEdit->text()), atoi(yEdit->text()),
                       atoi(wEdit->text()), atoi(hEdit->text()) );
  _target->setMinimumSize(atoi(minwEdit->text()), atoi(minhEdit->text()) );
  _target->setMaximumSize(atoi(maxwEdit->text()), atoi(maxhEdit->text()) );
}

void GeoPage::fixsize()
{
  minwEdit->setText(wEdit->text());
  maxwEdit->setText(wEdit->text());
  minhEdit->setText(hEdit->text());
  maxhEdit->setText(hEdit->text());
}
