#include "inspect.h"

/*!
  \fn WidgetInspector::target()

  Returns the current target of the WidgetInspector.
*/

/*!
  Create a new (empty) WidgetInspector.
*/

WidgetInspector::WidgetInspector(QWidget *parent, const char *name)
: QTabDialog(parent, name, TRUE)
{
  resize(450, 350);
  setCancelButton();
  setApplyButton();
  _pageList.setAutoDelete(TRUE);
}

/*!
  Set the target of the WidgetInspector. First, the whole QTabDialog will be
  cleared. Then, depending on the type of target, one or more pages will be
  added. Each page will the be called to set its target.
*/

void WidgetInspector::setTarget(QWidget *target)
{
  _target = target;
  _pageList.clear();

  PropPage *page;

  // Add pages for standard widget properties
  page = new MiscPage(this, "");
  _pageList.append(page);
  addTab(page, "Misc");
  page = new GeoPage(this, "");
  _pageList.append(page);
  addTab(page, "Geometry");
  page = new FontPage(this, "");
  _pageList.append(page);
  addTab(page, "Font");
  page = new ColorPage(this, "");
  _pageList.append(page);
  addTab(page, "Color");

  // Check for inheritance
  if(target->inherits("QButton"))
  {
    page = new ButtonPage(this, "");
    _pageList.append(page);
    addTab(page, "Button");
  }
  if(target->inherits("QFrame"))
  {
    page = new FramePage(this, "");
    _pageList.append(page);
    addTab(page, "Frame");
  }
  if(target->inherits("QGroupBox"))
  {
    page = new GroupBoxPage(this, "");
    _pageList.append(page);
    addTab(page, "GroupBox");
  }

  // Check the actual type
  QString cname = target->className();
  if(cname == "QCheckBox")
  {
    page = new CheckBoxPage(this, "");
    _pageList.append(page);
    addTab(page, "Check box");
  }
  else if(cname == "QPushButton")
  {
    page = new PushButtonPage(this, "");
    _pageList.append(page);
    addTab(page, "Push button");
  }
  else if(cname == "QRadioButton")
  {
    page = new RadioButtonPage(this, "");
    _pageList.append(page);
    addTab(page, "Radio button");
  }
  else if(cname == "QComboBox")
  {
    page = new ComboBoxPage(this, "");
    _pageList.append(page);
    addTab(page, "Combo box");
  }
  else if(cname == "QButtonGroup")
  {
    page = new ButtonGroupPage(this, "");
    _pageList.append(page);
    addTab(page, "Button group");
  }
  else if(cname == "QLabel")
  {
    page = new LabelPage(this, "");
    _pageList.append(page);
    addTab(page, "Label");
  }
  else if(cname == "QListBox")
  {
    page = new ListBoxPage(this, "");
    _pageList.append(page);
    addTab(page, "List box");
  }
  else if(cname == "QLineEdit")
  {
    page = new LineEditPage(this, "");
    _pageList.append(page);
    addTab(page, "Line edit");
  }
  else if(cname == "QMultiLineEdit")
  {
    page = new MultiLineEditPage(this, "");
    _pageList.append(page);
    addTab(page, "Multi line edit");
  }
  else if(cname == "QScrollBar")
  {
    page = new ScrollBarPage(this, "");
    _pageList.append(page);
    addTab(page, "Scroll bar");
  }
  else if(cname == "QSlider")
  {
    page = new SliderPage(this, "");
    _pageList.append(page);
    addTab(page, "Slider");
  }

  // Set the target of all pages in the list and connect SIGNALS
  QListIterator<PropPage> it(_pageList);
  for(; it.current(); ++it)
  {
    it.current()->setTarget(target);
    QObject::connect(this, SIGNAL(applyButtonPressed()),
                     it.current(), SLOT(applyValues()) );
  }
}
