#ifndef WidgetInspector_included
#define WidgetInspector_included

#include <qtabdlg.h>
#include <qlist.h>

#include <proppage.h>
#include <bttngrppage.h>
#include <bttnpage.h>
#include <chkboxpage.h>
#include <colmixer.h>
#include <colpage.h>
#include <combopage.h>
#include <fontpage.h>
#include <framepage.h>
#include <geopage.h>
#include <grpboxpage.h>
#include <labelpage.h>
#include <linedpage.h>
#include <listboxpage.h>
#include <miscpage.h>
#include <mlinedpage.h>
#include <pushbtpage.h>
#include <radiobtpage.h>
#include <scrbarpage.h>
#include <sliderpage.h>

/*!
  \class WidgetInspector inspect.h
  \brief Class used to display and modify a widget's properties

  The WidgetInspector is a smart class that can be used to display and
  modify the properties of any type of widget. It consists of a QTabDialog
  with tab pages for all of the standard widget properties and one or more
  extra tab pages for properties which are specific to a certain type of
  widget. As soon as the target of the WidgetInspector is set, its type is
  retrieved using QObject::className(). The target widget is also checked
  for it's inheritance structure. For example, if the target is a QGroupBox,
  which inherits QFrame, the WidgetInspector will have tab pages for both
  QFrame, containing properties like \e FrameStyle and QGroupBox, containing
  amoung others the title of the QGroupBox.

  \warning
  The WidgetInspector is a one-shot class at the moment. You can only set
  the target once. This is due to the simple fact that its base class
  QTabDialog does not provide a removeTab() member function. The best
  work-around for this problem is to create a new WidgetInspector for each
  object you wish to inspect and delete it when you are done with it.
*/

class WidgetInspector: public QTabDialog
{
  Q_OBJECT
  public:
    WidgetInspector(QWidget *parent=0, const char *name=0);
    void setTarget(QWidget *target);
    QWidget *target() {return _target;}
  private:
    //! The target of the widget inspector
    QWidget *_target;
    //! A list of pages created to view the content of the target widget
    QList<PropPage> _pageList;
};

#endif
