#include "labelpage.h"

LabelPage::LabelPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  textGrp = new QGroupBox(this);
  textGrp->setGeometry(0, 0, 310, 55);
  textGrp->setTitle("Text");
  textEdit = new QLineEdit(textGrp);
  textEdit->setGeometry(5, 20, 300, 25);
  halignGrp = new QGroupBox(this);
  halignGrp->setGeometry(0, 55, 150, 55);
  halignGrp->setTitle("Horizontal Alignment");
  halignCombo = new QComboBox(halignGrp);
  halignCombo->setGeometry(5, 20, 100, 25);
  halignCombo->insertItem("Left");
  halignCombo->insertItem("Right");
  halignCombo->insertItem("Center");
  valignGrp = new QGroupBox(this);
  valignGrp->setGeometry(155, 55, 155, 55);
  valignGrp->setTitle("Vertical Alignment");
  valignCombo = new QComboBox(valignGrp);
  valignCombo->setGeometry(5, 20, 100, 25);
  valignCombo->insertItem("Top");
  valignCombo->insertItem("Bottom");
  valignCombo->insertItem("Center");
  optGrp = new QGroupBox(this);
  optGrp->setGeometry(0, 110, 310, 55);
  optGrp->setTitle("Options");
  untitled_0 = new QLabel(optGrp);
  untitled_0->setGeometry(5, 20, 65, 25);
  untitled_0->setText("Margin");
  marginEdit = new QLineEdit(optGrp);
  marginEdit->setGeometry(70, 20, 80, 25);
  aresBox = new QCheckBox(optGrp);
  aresBox->setGeometry(195, 20, 100, 30);
  aresBox->setText("Auto Resize");

  connect(textEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(halignCombo, SIGNAL(activated(int)), SLOT(setDirty(int)) );
  connect(valignCombo, SIGNAL(activated(int)), SLOT(setDirty(int)) );
  connect(marginEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
}

LabelPage::~LabelPage()
{
}

void LabelPage::setTarget(QWidget *target)
{
  _target = (QLabel *)target;
  QString tmpStr(_target->text());
  tmpStr.replace(QRegExp("\n"), "\\n");
  textEdit->setText(tmpStr);
  int amode = _target->alignment();
  if(amode & AlignLeft)
    halignCombo->setCurrentItem(0);
  else if(amode & AlignRight)
    halignCombo->setCurrentItem(1);
  else if(amode & AlignHCenter)
    halignCombo->setCurrentItem(2);
  if(amode & AlignTop)
    valignCombo->setCurrentItem(0);
  else if(amode & AlignBottom)
    valignCombo->setCurrentItem(1);
  else if(amode & AlignVCenter)
    valignCombo->setCurrentItem(2);
  QString buffer;
  buffer.setNum(_target->margin());
  marginEdit->setText(buffer);
  aresBox->setChecked(_target->autoResize());
  dirty = 0;
}

void LabelPage::applyValues()
{
  if(! dirty)
    return;
  QString tmpStr(textEdit->text());
  int idx = tmpStr.find("\\n");
  while(idx != -1)
  {
    tmpStr = tmpStr.left(idx) + '\n'
             + tmpStr.right(tmpStr.length()-idx-2);
    idx = tmpStr.find("\\n", idx);
  }
  _target->setText(tmpStr);
  _target->setMargin(atoi(marginEdit->text()));
  int align = 0;
  switch(halignCombo->currentItem())
  {
    case 0:
      align |= AlignLeft;
      break;
    case 1:
      align |= AlignRight;
      break;
    case 2:
      align |= AlignHCenter;
      break;
  }
  switch(valignCombo->currentItem())
  {
    case 0:
      align |= AlignTop;
      break;
    case 1:
      align |= AlignBottom;
      break;
    case 2:
      align |= AlignVCenter;
      break;
  }
  _target->setAlignment(align);
  _target->setAutoResize(aresBox->isChecked());
}
