#include "linedpage.h"

LineEditPage::LineEditPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  textGrp = new QGroupBox(this);
  textGrp->setGeometry(0, 0, 160, 55);
  textGrp->setTitle("Text");
  textEdit = new QLineEdit(textGrp);
  textEdit->setGeometry(5, 20, 150, 25);
  modeGrp = new QGroupBox(this);
  modeGrp->setGeometry(165, 0, 140, 55);
  modeGrp->setTitle("Echo Mode");
  modeCombo = new QComboBox(modeGrp);
  modeCombo->setGeometry(20, 20, 100, 25);
  modeCombo->insertItem("Normal");
  modeCombo->insertItem("NoEcho");
  modeCombo->insertItem("Password");
  lenGrp = new QGroupBox(this);
  lenGrp->setGeometry(0, 55, 160, 55);
  lenGrp->setTitle("Maximum Length");
  lenEdit = new QLineEdit(lenGrp);
  lenEdit->setGeometry(25, 20, 100, 25);
  optGrp = new QGroupBox(this);
  optGrp->setGeometry(165, 55, 140, 55);
  optGrp->setTitle("Options");
  frameBox = new QCheckBox(optGrp);
  frameBox->setGeometry(20, 20, 100, 25);
  frameBox->setText("Frame");

  connect(textEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char*)) );
  connect(modeCombo, SIGNAL(activated(int)), SLOT(setDirty(int)) );
  connect(lenEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char*)) );
  connect(frameBox, SIGNAL(clicked()), SLOT(setDirty()) );
}

LineEditPage::~LineEditPage()
{
}

void LineEditPage::setTarget(QWidget *target)
{
  _target = (QLineEdit *)target;
  textEdit->setText(_target->text());
  modeCombo->setCurrentItem((int)_target->echoMode());
  QString buffer;
  buffer.setNum(_target->maxLength());
  lenEdit->setText(buffer);
  frameBox->setChecked(_target->frame());
  dirty = 0;
}

void LineEditPage::applyValues()
{
  if(! dirty)
    return;
  _target->setText(textEdit->text());
  _target->setEchoMode((QLineEdit::EchoMode)modeCombo->currentItem());
  _target->setMaxLength(atoi(lenEdit->text()));
  _target->setFrame(frameBox->isChecked());
}
