#include "listboxpage.h"

ListBoxPage::ListBoxPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  itemGrp = new QGroupBox(this);
  itemGrp->setGeometry(0, 0, 310, 110);
  itemGrp->setTitle("Items");
  itemEdit = new QLineEdit(itemGrp);
  itemEdit->setGeometry(5, 20, 145, 25);
  addBut = new QPushButton(itemGrp);
  addBut->setGeometry(5, 50, 70, 25);
  addBut->setText("Add");
  insBut = new QPushButton(itemGrp);
  insBut->setGeometry(80, 50, 70, 25);
  insBut->setText("Insert");
  chgBut = new QPushButton(itemGrp);
  chgBut->setGeometry(5, 80, 70, 25);
  chgBut->setText("Change");
  delBut = new QPushButton(itemGrp);
  delBut->setGeometry(80, 80, 70, 25);
  delBut->setText("Delete");
  itemList = new QListBox(itemGrp);
  itemList->setGeometry(155, 20, 150, 85);
  optGrp = new QGroupBox(this);
  optGrp->setGeometry(0, 110, 310, 80);
  optGrp->setTitle("Options");
  ascrBox = new QCheckBox(optGrp);
  ascrBox->setGeometry(5, 20, 130, 25);
  ascrBox->setText("Auto Scroll");
  dragBox = new QCheckBox(optGrp);
  dragBox->setGeometry(155, 20, 130, 25);
  dragBox->setText("Drag Selection");
  ascrbarBox = new QCheckBox(optGrp);
  ascrbarBox->setGeometry(5, 45, 130, 25);
  ascrbarBox->setText("Auto Scrollbar");
  smoothBox = new QCheckBox(optGrp);
  smoothBox->setGeometry(155, 45, 130, 25);
  smoothBox->setText("Smooth Scrolling");

  connect(addBut, SIGNAL(clicked()), SLOT(addEntry()) );
  connect(addBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(insBut, SIGNAL(clicked()), SLOT(insEntry()) );
  connect(insBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(chgBut, SIGNAL(clicked()), SLOT(chgEntry()) );
  connect(chgBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(delBut, SIGNAL(clicked()), SLOT(delEntry()) );
  connect(delBut, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(itemList, SIGNAL(highlighted(int)), SLOT(newItem(int)) );
  connect(itemEdit, SIGNAL(returnPressed()), SLOT(addEntry()) );
  connect(itemEdit, SIGNAL(returnPressed()), SLOT(setDirty()) );
}

ListBoxPage::~ListBoxPage()
{
}

void ListBoxPage::setTarget(QWidget *target)
{
  _target = (QListBox *)target;
  for(int i=0; i<(int)_target->count(); i++)
    itemList->insertItem(_target->text(i));
  itemEdit->setText("");
  ascrBox->setChecked(_target->autoScroll());
  ascrbarBox->setChecked(_target->autoScrollBar());
  dragBox->setChecked(_target->dragSelect());
  smoothBox->setChecked(_target->smoothScrolling());
  dirty = 0;
}

void ListBoxPage::applyValues()
{
  if(! dirty)
    return;
  _target->clear();
  for(int i=0; i<(int)itemList->count(); i++)
    _target->insertItem(itemList->text(i));
  _target->setAutoScroll(ascrBox->isChecked());
  _target->setAutoScrollBar(ascrbarBox->isChecked());
  _target->setDragSelect(dragBox->isChecked());
  _target->setSmoothScrolling(smoothBox->isChecked());
}

void ListBoxPage::addEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  itemList->insertItem(itemEdit->text());
}

void ListBoxPage::insEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  if(itemList->currentItem() >= 0)
    itemList->insertItem(itemEdit->text(), itemList->currentItem());
  else
    itemList->insertItem(itemEdit->text());
}

void ListBoxPage::chgEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  if(itemList->currentItem() >= 0)
    itemList->changeItem(itemEdit->text(), itemList->currentItem());
}

void ListBoxPage::delEntry()
{
  if(*itemEdit->text() == '\0')
    return;
  if(itemList->currentItem() >= 0)
    itemList->removeItem(itemList->currentItem());
}

void ListBoxPage::newItem(int idx)
{
  itemEdit->setText(itemList->text(idx));
}
