#include "mflmdl.h"

MemberFuncListModule::MemberFuncListModule(QWidget *parent, const char *name)
: QWidget(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 235);
  this->setCaption("Member Function List");
  backFrame = new QFrame(this);
  backFrame->setGeometry(5, 10, 390, 220);
  backFrame->setFrameStyle(49);
  typeList = new QListBox(backFrame);
  typeList->setGeometry(10, 30, 100, 140);
  nameList = new QListBox(backFrame);
  nameList->setGeometry(110, 30, 150, 140);
  untitled_0 = new QLabel(backFrame);
  untitled_0->setGeometry(10, 5, 100, 25);
  untitled_0->setText("Return type");
  untitled_0->setAlignment(36);
  untitled_1 = new QLabel(backFrame);
  untitled_1->setGeometry(110, 5, 150, 25);
  untitled_1->setText("Name");
  untitled_1->setAlignment(36);
  newBut = new QPushButton(backFrame);
  newBut->setGeometry(15, 180, 100, 25);
  newBut->setText("New");
  rmBut = new QPushButton(backFrame);
  rmBut->setGeometry(140, 180, 100, 25);
  rmBut->setText("Remove");
  editBut = new QPushButton(backFrame);
  editBut->setGeometry(265, 180, 100, 25);
  editBut->setText("Edit");
  untitled_2 = new QLabel(backFrame);
  untitled_2->setGeometry(260, 5, 100, 25);
  untitled_2->setText("Access");
  untitled_2->setAlignment(36);
  accessList = new QListBox(backFrame);
  accessList->setGeometry(260, 30, 110, 140);

  GeoMgr *manager = new GeoMgr(this);
  manager->init();
  manager->setLayoutInfo(backFrame, NW_GRAV, SE_GRAV, RESIZE_ALL);
  GeoMgr *submgr = new GeoMgr(backFrame);
  submgr->init();
  submgr->setLayoutInfo(untitled_0, N_GRAV, N_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(untitled_1, N_GRAV, N_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(untitled_2, N_GRAV, N_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(typeList, NW_GRAV, S_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(nameList, N_GRAV, S_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(accessList, N_GRAV, SE_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(newBut, S_GRAV, S_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(rmBut, S_GRAV, S_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(editBut, S_GRAV, S_GRAV, RESIZE_NONE);
  connect(typeList, SIGNAL(highlighted(int)), SLOT(syncLists(int)) );
  connect(typeList, SIGNAL(selected(int)), SLOT(editMember(int)) );
  connect(nameList, SIGNAL(highlighted(int)), SLOT(syncLists(int)) );
  connect(nameList, SIGNAL(selected(int)), SLOT(editMember(int)) );
  connect(accessList, SIGNAL(highlighted(int)), SLOT(syncLists(int)) );
  connect(accessList, SIGNAL(selected(int)), SLOT(editMember(int)) );
  connect(newBut, SIGNAL(clicked()), SLOT(newMember()) );
  connect(rmBut, SIGNAL(clicked()), SLOT(delMember()) );
  connect(editBut, SIGNAL(clicked()), SLOT(editMember()) );
}

MemberFuncListModule::~MemberFuncListModule()
{
}

void MemberFuncListModule::setList(QList<MemberFunction> &lst)
{
  list = &lst;
  update();
}

void MemberFuncListModule::closeEvent(QCloseEvent *ev)
{
  ev->ignore();
  QMessageBox::warning(this, "Ebuilder",
    "Use the window menu item to close this window");
}

void MemberFuncListModule::syncLists(int idx)
{
  typeList->setCurrentItem(idx);
  nameList->setCurrentItem(idx);
  accessList->setCurrentItem(idx);
}

void MemberFuncListModule::newMember()
{
  MemberFunction *entry = new MemberFunction;
  entry->type = "";
  entry->name = "";
  entry->arglist = "";
  entry->access = "public";
  entry->extra = "";
  entry->text = "";
  MemberFuncModal tmp(0, "");
  tmp.setMember(entry);
  tmp.exec();
  if(tmp.result())
  {
    list->append(entry);
    update();
  }
}

void MemberFuncListModule::delMember()
{
  int idx = typeList->currentItem();
  if(idx < 0)
    return;
  list->remove(idx);
  typeList->removeItem(idx);
  nameList->removeItem(idx);
  update();
}

void MemberFuncListModule::editMember()
{
  int idx = typeList->currentItem();
  if(idx < 0)
    return;
  QListIterator<MemberFunction> it(*list);
  for(int i=0; i<idx; i++)
    ++it;
  MemberFuncModal tmp(0, "");
  MemberFunction *entry = it.current();
  tmp.setMember(entry);
  tmp.exec();
  if(tmp.result())
    update();
}

void MemberFuncListModule::editMember(int)
{
  editMember();
}

void MemberFuncListModule::update()
{
  typeList->clear();
  nameList->clear();
  accessList->clear();
  QListIterator<MemberFunction> it(*list);
  for(; it.current(); ++it)
  {
    MemberFunction *curr = it.current();
    typeList->insertItem(curr->type);
    nameList->insertItem(curr->name);
    QString tmp(curr->access);
    tmp = tmp + " " + curr->extra;
    accessList->insertItem(tmp);
  }
}
