#include "miscpage.h"

MiscPage::MiscPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  styleGrp = new QGroupBox(this);
  styleGrp->setGeometry(0, 0, 120, 55);
  styleGrp->setTitle("GUI Style");
  styleCombo = new QComboBox(styleGrp);
  styleCombo->setGeometry(5, 20, 110, 25);
  styleCombo->insertItem("MacStyle");
  styleCombo->insertItem("WindowsStyle");
  styleCombo->insertItem("Win3Style");
  styleCombo->insertItem("PMStyle");
  styleCombo->insertItem("MotifStyle");
  focusGrp = new QGroupBox(this);
  focusGrp->setGeometry(125, 0, 110, 55);
  focusGrp->setTitle("Focus Policy");
  focusCombo = new QComboBox(focusGrp);
  focusCombo->setGeometry(5, 20, 100, 25);
  focusCombo->insertItem("NoFocus");
  focusCombo->insertItem("TabFocus");
  focusCombo->insertItem("ClickFocus");
  focusCombo->insertItem("StrongFocus");
  optGrp = new QGroupBox(this);
  optGrp->setGeometry(0, 55, 235, 105);
  optGrp->setTitle("Options");
  mousetrBox = new QCheckBox(optGrp);
  mousetrBox->setGeometry(5, 20, 150, 25);
  mousetrBox->setText("Mouse Tracking");
  enableBox = new QCheckBox(optGrp);
  enableBox->setGeometry(5, 45, 150, 25);
  enableBox->setText("Enable Widget");
  captionEdit = new QLineEdit(optGrp);
  captionEdit->setGeometry(60, 70, 165, 25);
  untitled_0 = new QLabel(optGrp);
  untitled_0->setGeometry(5, 70, 50, 25);
  untitled_0->setText("Caption");

  connect(styleCombo, SIGNAL(activated(int)), SLOT(setDirty(int)) );
  connect(mousetrBox, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(enableBox, SIGNAL(clicked()), SLOT(setDirty()) );
  connect(focusCombo, SIGNAL(activated(int)), SLOT(setDirty(int)) );
  connect(captionEdit, SIGNAL(textChanged(const char *))
  , SLOT(setDirty(const char *)) );
}

MiscPage::~MiscPage()
{
}

void MiscPage::setTarget(QWidget *target)
{
 _target = target;
 styleCombo->setCurrentItem(_target->style());
 focusCombo->setCurrentItem(_target->focusPolicy());
 mousetrBox->setChecked(_target->hasMouseTracking());
 enableBox->setChecked(_target->isEnabled());
 if(_target->parent() == 0)
 {
   captionEdit->setEnabled(TRUE);
   captionEdit->setText(_target->caption());
 }
 else
 {
   captionEdit->setEnabled(FALSE);
   captionEdit->setText("");
 }
  dirty = 0;
}

void MiscPage::applyValues()
{
  if(! dirty)
    return;
  _target->setStyle((GUIStyle)styleCombo->currentItem());
  _target->setFocusPolicy((QWidget::FocusPolicy)focusCombo->currentItem());
  _target->setMouseTracking(mousetrBox->isChecked());
  _target->setEnabled(enableBox->isChecked());
  if(_target->parent() == 0)
    _target->setCaption(captionEdit->text());
}
