#include "mvlmdl.h"

MemberVarListModule::MemberVarListModule(QWidget *parent, const char *name)
: QWidget(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(430, 250);
  this->setCaption("Member Variable List");
  backFrame = new QFrame(this);
  backFrame->setGeometry(5, 10, 420, 230);
  backFrame->setFrameStyle(49);
  untitled_0 = new QLabel(backFrame);
  untitled_0->setGeometry(10, 15, 150, 25);
  untitled_0->setText("Type");
  untitled_0->setAlignment(36);
  typeList = new QListBox(backFrame);
  typeList->setGeometry(10, 40, 150, 175);
  untitled_1 = new QLabel(backFrame);
  untitled_1->setGeometry(160, 15, 140, 25);
  untitled_1->setText("Name");
  untitled_1->setAlignment(36);
  nameList = new QListBox(backFrame);
  nameList->setGeometry(160, 40, 140, 175);
  newBut = new QPushButton(backFrame);
  newBut->setGeometry(310, 80, 100, 25);
  newBut->setText("New");
  rmBut = new QPushButton(backFrame);
  rmBut->setGeometry(310, 115, 100, 25);
  rmBut->setText("Remove");
  editBut = new QPushButton(backFrame);
  editBut->setGeometry(310, 150, 100, 25);
  editBut->setText("Edit");

  GeoMgr *manager = new GeoMgr(this);
  manager->init();
  manager->setLayoutInfo(backFrame, NW_GRAV, SE_GRAV, RESIZE_ALL);
  GeoMgr *submgr = new GeoMgr(backFrame);
  submgr->init();
  submgr->setLayoutInfo(untitled_0, N_GRAV, N_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(untitled_1, N_GRAV, N_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(typeList, NW_GRAV, S_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(nameList, N_GRAV, SE_GRAV, RESIZE_ALL);
  submgr->setLayoutInfo(newBut, E_GRAV, E_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(rmBut, E_GRAV, E_GRAV, RESIZE_NONE);
  submgr->setLayoutInfo(editBut, E_GRAV, E_GRAV, RESIZE_NONE);
  connect(typeList, SIGNAL(highlighted(int)), SLOT(syncLists(int)) );
  connect(typeList, SIGNAL(selected(int)), SLOT(editVar(int)) );
  connect(nameList, SIGNAL(highlighted(int)), SLOT(syncLists(int)) );
  connect(nameList, SIGNAL(selected(int)), SLOT(editVar(int)) );
  connect(newBut, SIGNAL(clicked()), SLOT(newVar()) );
  connect(rmBut, SIGNAL(clicked()), SLOT(delVar()) );
  connect(editBut, SIGNAL(clicked()), SLOT(editVar()) );
}

MemberVarListModule::~MemberVarListModule()
{
}

void MemberVarListModule::setList(QList<MemberVariable> &lst)
{
  list = &lst;
  update();
}

void MemberVarListModule::closeEvent(QCloseEvent *ev)
{
  ev->ignore();
  QMessageBox::warning(this, "Ebuilder",
    "Use the window menu item to close this window");
}

void MemberVarListModule::syncLists(int idx)
{
  typeList->setCurrentItem(idx);
  nameList->setCurrentItem(idx);
}

void MemberVarListModule::newVar()
{
  MemberVariable *entry = new MemberVariable;
  entry->type = "";
  entry->name = "";
  MemberVarModal tmp(0, "");
  tmp.setVar(entry);
  tmp.exec();
  if(tmp.result())
  {
    list->append(entry);
    update();
  }
}

void MemberVarListModule::delVar()
{
  int idx = typeList->currentItem();
  if(idx < 0)
    return;
  list->remove(idx);
  typeList->removeItem(idx);
  nameList->removeItem(idx);
  update();
}

void MemberVarListModule::editVar()
{
  int idx = typeList->currentItem();
  if(idx < 0)
    return;
  QListIterator<MemberVariable> it(*list);
  for(int i=0; i<idx; i++)
    ++it;
  MemberVarModal tmp(0, "");
  MemberVariable *entry = it.current();
  tmp.setVar(entry);
  tmp.exec();
  if(tmp.result())
    update();
}

void MemberVarListModule::editVar(int)
{
  editVar();
}

void MemberVarListModule::update()
{
  typeList->clear();
  nameList->clear();
  QListIterator<MemberVariable> it(*list);
  for(; it.current(); ++it)
  {
    MemberVariable *curr = it.current();
    typeList->insertItem(curr->type);
    nameList->insertItem(curr->name);
  }
}
