%{
#include <iostream.h>
#include <qstring.h>
#include <qregexp.h>
#include "storemdl.h"
%}

WORD [A-Za-z]+
WPROP_A Geometry|Font|Palette|GUIStyle|FocusPolicy|MouseTracking|Caption|Enable
WPROP_B AutoResize|AutoRepeat|Text|Checked|ToggleButton|Item|SizeLimit
WPROP_C FrameStyle|LineWidth|MidLineWidth|Title|Alignment|Exclusive
WPROP_D HasFrame|EchoMode|ReadOnly|OverWriteMode|DragSelect|AutoScroll
WPROP_E AutoScrollBar|SmoothScroll|Range|Steps|Value|Orientation|Tracking
WPROP_F Tickmarks|TickInterval|Margin|TextLine|Default|AutoDefault
WPROP_G Arguments|BaseClass|BaseInit|BaseHeader|Modal
WPROP {WPROP_A}|{WPROP_B}|{WPROP_C}|{WPROP_D}|{WPROP_E}|{WPROP_F}|{WPROP_G}

MEMF MemberFunction
MFPROP RetType|Name|ArgList|Access|Extra|CodeLine

MEMV MemberVariable
MVPROP Type|Name|Access

INCLUDE Include
INCPROP Name

BASETYPE QWidget|QDialog|QMDialog|QFrame
SUBTYPE  QCheckBox|QPushButton|QRadioButton|QComboBox|QFrame|QGroupBox|QButtonGroup|QLabel|QMenuBar|QListBox|QMultiLineEdit|QLineEdit|QScrollBar|QSlider

TYPE {BASETYPE}|{SUBTYPE}
NAME \[[^\]]*\]
PROP {WPROP}|{MFPROP}|{MVPROP}|{INCPROP}
VALUE \{.*\}\n

%x GetName
%x GetValue

%%
%{
%}

<*># {
  // Everything after a hash is treated as comment
  int c;
  while( (c = yyinput()) != '\n' && c != EOF );
  parseObj->lineNumber++;
  unput(c);
}

<*>[ \t]*  /* Whitespace is ignored */

<*>\n { 
  // Newline characters increments the line number
  parseObj->lineNumber++; 
}

{TYPE} {
  //cerr << parseObj->lineNumber << ": Found type: " << yytext << endl;
  parseObj->type = yytext;
  BEGIN(GetName);
}

{PROP} {
  //cerr << parseObj->lineNumber << ": Found property: " << yytext << endl;
  parseObj->key = yytext;
  BEGIN(GetValue);
}

{MEMF} {
  //cerr <<  parseObj->lineNumber << ": Found member function" << endl;
  return (int)MemberFuncStart;
}

{MEMV} {
  //cerr <<  parseObj->lineNumber << ": Found member variable" << endl;
  return (int)MemberVarStart;
}

{INCLUDE} {
  //cerr <<  parseObj->lineNumber << ": Found include" << endl;
  return (int)IncludeStart;
}

\{ {
  //cerr << parseObj->lineNumber << ": Found open bracket" << endl;
  return (int)OpenBracket;
}

\} {
  //cerr << parseObj->lineNumber << ": Found close bracket" << endl;
  return (int)CloseBracket;
}

. {
  //cerr << parseObj->lineNumber << ": Bad symbol: " << yytext << endl;
  return (int)ParseError;
}

<GetName>{NAME} {
  //cerr << parseObj->lineNumber << ": Found name: " << yytext << endl;
  // Extract name value
  QString str(yytext);
  str = str.mid(1, str.length()-2);
  parseObj->name = str;
  BEGIN(0);
  return (int)WidgetStart;
}

<GetName>. {
  //cerr << parseObj->lineNumber << ": Bad symbol: " << yytext << endl;
  BEGIN(0);
  return (int)ParseError;
}

<GetValue>{VALUE} {
  //cerr << parseObj->lineNumber << ": Found value: " << yytext << endl;
  // Extract the value
  QString str(yytext);
  str = str.mid(1, str.length()-3);
  parseObj->value = str;
  parseObj->lineNumber++;
  BEGIN(0);
  return (int)PropertyReady;
}

<GetValue>. {
  //cerr << parseObj->lineNumber << ": Bad symbol: " << yytext << endl;
  BEGIN(0);
  return (int)ParseError;
}

%%
