#include "prefmdl.h"

PrefModule::PrefModule(QWidget *parent, const char *name)
: QDialog(parent, name, TRUE)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(310, 220);
  this->setCaption("Preferences");
  expGrp = new QGroupBox(this);
  expGrp->setGeometry(0, 0, 310, 105);
  expGrp->setTitle("Export properties");
  untitled_7 = new QLabel(expGrp);
  untitled_7->setGeometry(20, 45, 50, 25);
  untitled_7->setText("Header");
  untitled_7->setAlignment(33);
  hextEdit = new QLineEdit(expGrp);
  hextEdit->setGeometry(70, 45, 50, 25);
  hextEdit->setText(".h");
  untitled_9 = new QLabel(expGrp);
  untitled_9->setGeometry(130, 45, 50, 25);
  untitled_9->setText("Impl.");
  untitled_9->setAlignment(33);
  iextEdit = new QLineEdit(expGrp);
  iextEdit->setGeometry(175, 45, 50, 25);
  iextEdit->setText(".cc");
  untitled_11 = new QLabel(expGrp);
  untitled_11->setGeometry(5, 75, 70, 25);
  untitled_11->setText("Directory");
  untitled_11->setAlignment(33);
  exdirEdit = new QLineEdit(expGrp);
  exdirEdit->setGeometry(70, 75, 170, 25);
  exdirEdit->setText(".");
  untitled_101 = new QLabel(expGrp);
  untitled_101->setGeometry(5, 20, 100, 25);
  untitled_101->setText("Extensions:");
  untitled_101->setAlignment(33);
  dsgnGrp = new QGroupBox(this);
  dsgnGrp->setGeometry(0, 105, 310, 80);
  dsgnGrp->setTitle("Designer Properties");
  untitled_16 = new QLabel(dsgnGrp);
  untitled_16->setGeometry(5, 20, 60, 25);
  untitled_16->setText("Grid");
  untitled_16->setAlignment(33);
  xgridEdit = new QLineEdit(dsgnGrp);
  xgridEdit->setGeometry(70, 20, 60, 25);
  xgridEdit->setText("5");
  untitled_18 = new QLabel(dsgnGrp);
  untitled_18->setGeometry(130, 20, 30, 25);
  untitled_18->setText("X");
  untitled_18->setAlignment(36);
  ygridEdit = new QLineEdit(dsgnGrp);
  ygridEdit->setGeometry(155, 20, 60, 25);
  ygridEdit->setText("5");
  untitled_52 = new QLabel(dsgnGrp);
  untitled_52->setGeometry(5, 50, 100, 25);
  untitled_52->setText("Default Style");
  untitled_52->setAlignment(33);
  styleCombo = new QComboBox(dsgnGrp);
  styleCombo->setGeometry(105, 50, 100, 25);
  styleCombo->setFocusPolicy((QWidget::FocusPolicy)3);
  styleCombo->insertItem("WindowsStyle");
  styleCombo->insertItem("MotifStyle");
  okBut = new QPushButton(this);
  okBut->setGeometry(55, 190, 80, 25);
  okBut->setText("&Ok");
  okBut->setDefault((bool)1);
  cancelBut = new QPushButton(this);
  cancelBut->setGeometry(140, 190, 80, 25);
  cancelBut->setText("&Cancel");
  saveBut = new QPushButton(this);
  saveBut->setGeometry(225, 190, 80, 25);
  saveBut->setText("&Save");

  connect(cancelBut, SIGNAL(clicked()), SLOT(reject()) );
  connect(okBut, SIGNAL(clicked()), SLOT(accept()) );
  connect(saveBut, SIGNAL(clicked()), SLOT(savePrefs()) );
}

PrefModule::~PrefModule()
{
}

void PrefModule::save(const char *fname)
{
  ofstream fout;
  fout.open(fname);
  if(! fout.good())
  {
    QMessageBox::critical(0, "Ebuilder", "Unable to open resouce file");
    return;
  }
  
  // Write all properties known to the PrefModule to disk
  fout << "HeaderExtension " << hextEdit->text() << endl;
  fout << "ImplExtension " << iextEdit->text() << endl;
  fout << "ExportDir" << exdirEdit->text() << endl;
  fout << "Grid " << xgridEdit->text() << " " << ygridEdit->text() << endl;
  fout << "DefaultStyle " << styleCombo->currentText() << endl;
  fout.close();
}

void PrefModule::init()
{
  rcFilename = getenv("EBUIDERRC");
  if(! rcFilename)
    rcFilename = getenv("HOME") + QString("/") + QString(".ebuilderrc");
  ifstream fin;
  fin.open(rcFilename);
  if(! fin.good())
    return;
  while(!fin.eof())
  {
    char buffer[1024];
    fin >> buffer;
    QString key = buffer;
    if(key == "HeaderExtension")
    {
      fin >> buffer;
      hextEdit->setText(buffer);
    }
    else if(key == "ImplExtension")
    {
      fin >> buffer;
      iextEdit->setText(buffer);
    }
    else if(key == "ExportDir")
    {
      fin >> buffer;
      exdirEdit->setText(buffer);
    }
    else if(key == "Grid")
    {
      fin >> buffer;
      xgridEdit->setText(buffer);
      fin >> buffer;
      ygridEdit->setText(buffer);
    }
    else if(key == "DefaultStyle")
    {
      fin >> buffer;
      QString tmp = buffer;
      if(tmp == "WindowsStyle")
        styleCombo->setCurrentItem(0);
      else
        styleCombo->setCurrentItem(1);
    }   
  }
  fin.close();
}

const char * PrefModule::headerExt()
{
  return hextEdit->text();
}

const char * PrefModule::implExt()
{
  return iextEdit->text();
}

const char * PrefModule::exportDir()
{
  return exdirEdit->text();
}

int PrefModule::xGrid()
{
  return atoi(xgridEdit->text());
}

int PrefModule::yGrid()
{
  return atoi(ygridEdit->text());
}

GUIStyle PrefModule::defaultStyle()
{
  if(styleCombo->currentItem() == 0)
    return WindowsStyle;
  else
    return MotifStyle;
}

void PrefModule::savePrefs()
{
  ValModal tmp(0, "");
  tmp.setQuestion("Filename of resource file to save to");
  tmp.setText(rcFilename);
  tmp.showAtCursor();
  if(tmp.result())
  {
    rcFilename = tmp.text();
    save(tmp.text());
    QMessageBox::information(this, "Ebuilder", "Options saved\n"
                             "You must re-load your current design\n"
                             "to apply default style changes!");
  }
}
