#ifndef PropPage_included
#define PropPage_included

#include <qwidget.h>

/*!
  \class PropPage inspect.h
  \brief The abstract base class for the property pages of the
         WidgetInspector class

  The PropPage class provides an abstract baseclass for the property pages
  in the widget inspector. All it really does is maintain a dirty flag to
  see if the content of the page was changed.
*/

class PropPage: public QWidget
{
  Q_OBJECT
  public:
    PropPage(QWidget *parent=0, const char *name=0)
    :QWidget(parent, name) {dirty = 0;}
    virtual void setTarget(QWidget *target) = 0;
  protected slots:
    virtual void applyValues() = 0;
    virtual void setDirty(const char *) {dirty = 1;}
    virtual void setDirty(int) {dirty = 1;}
    virtual void setDirty() {dirty = 1;}
  protected:
    //! Flag to see whether any of the properties on the page were changed
    int dirty;
};

/*!
  \fn PropPage::PropPage(QWidget *parent=0, const char *name=0)

  Property page constructor. Should be called by derived class.
*/

/*!
  \fn virtual void PropPage::setTarget(QWidget *target)

  Pure virtual function to set the target of the property page. The
  implementation in the derived class should extract the properties from the
  widget and display them on the page.
*/

/*!
  \fn virtual void PropPage::applyValues()

  Pure virtual function that applies the changes made to the properties of
  the target widget. The implementation should check the dirty flag to see
  if changes should be made to the target.
*/

/*!
  \fn virtual void PropPage::setDirty(const char *)

  Sets the dirty flag. This slot should be connected to QLineEdit-s on the
  property page.
*/

/*!
  \fn virtual void PropPage::setDirty(int)

  Sets the dirty flag. Overloaded member function. This slot should be
  connected to QComboBox-s and such on the property page.
*/

/*!
  \fn virtual void PropPage::setDirty()

  Sets the dirty flag. Overloaded member function. This slot should be
  connected to QPushButton-s and such on the property page.
*/

#endif
