#include "scrbarpage.h"

ScrollBarPage::ScrollBarPage(QWidget *parent, const char *name)
: PropPage(parent, name)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(400, 300);
  rangeGrp = new QGroupBox(this);
  rangeGrp->setGeometry(0, 0, 310, 55);
  rangeGrp->setTitle("Range (min, max, value)");
  minEdit = new QLineEdit(rangeGrp);
  minEdit->setGeometry(5, 20, 70, 25);
  maxEdit = new QLineEdit(rangeGrp);
  maxEdit->setGeometry(80, 20, 70, 25);
  valEdit = new QLineEdit(rangeGrp);
  valEdit->setGeometry(155, 20, 70, 25);
  stepGrp = new QGroupBox(this);
  stepGrp->setGeometry(0, 55, 310, 55);
  stepGrp->setTitle("Steps (line, page)");
  lstepEdit = new QLineEdit(stepGrp);
  lstepEdit->setGeometry(5, 20, 70, 25);
  pstepEdit = new QLineEdit(stepGrp);
  pstepEdit->setGeometry(80, 20, 70, 25);
  optGrp = new QGroupBox(this);
  optGrp->setGeometry(0, 110, 310, 55);
  optGrp->setTitle("Options");
  orientCombo = new QComboBox(optGrp);
  orientCombo->setGeometry(5, 20, 100, 25);
  orientCombo->insertItem("Horizontal");
  orientCombo->insertItem("Vertical");
  trackBox = new QCheckBox(optGrp);
  trackBox->setGeometry(115, 20, 100, 25);
  trackBox->setText("Tracking");

  connect(minEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(maxEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(valEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(lstepEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(pstepEdit, SIGNAL(textChanged(const char *))
          , SLOT(setDirty(const char *)) );
  connect(orientCombo, SIGNAL(activated(int)), SLOT(setDirty(int)) );
  connect(trackBox, SIGNAL(clicked()), SLOT(setDirty()) );
}

ScrollBarPage::~ScrollBarPage()
{
}

void ScrollBarPage::setTarget(QWidget *target)
{
  _target = (QScrollBar *)target;
  QString buffer;
  buffer.setNum(_target->minValue());
  minEdit->setText(buffer);
  buffer.setNum(_target->maxValue());
  maxEdit->setText(buffer);
  buffer.setNum(_target->lineStep());
  valEdit->setText(buffer);
  lstepEdit->setText(buffer);
  buffer.setNum(_target->pageStep());
  pstepEdit->setText(buffer);
  buffer.setNum(_target->value());
  orientCombo->setCurrentItem((int)_target->orientation());
  trackBox->setChecked(_target->tracking());
  dirty = 0;
}

void ScrollBarPage::applyValues()
{
  if(! dirty)
    return;
  _target->setRange(atoi(minEdit->text()), atoi(maxEdit->text()) );
  _target->setValue(atoi(valEdit->text()));
  _target->setSteps(atoi(lstepEdit->text()), atoi(pstepEdit->text()));
  _target->setOrientation((QScrollBar::Orientation)orientCombo->currentItem());
  _target->setTracking(trackBox->isChecked());
}
