#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>

#include <qdialog.h>
#include <qchkbox.h>
#include <qpushbt.h>
#include <qradiobt.h>
#include <qcombo.h>
#include <qframe.h>
#include <qgrpbox.h>
#include <qbttngrp.h>
#include <qlined.h>
#include <qmlined.h>
#include <qlabel.h>
#include <qscrbar.h>
#include <qslider.h>
#include <qlistbox.h>
#include <qmenubar.h>

#include <qobjcoll.h>
#include <qmsgbox.h>
#include <qregexp.h>
#include <qfile.h>

#include "storemdl.h"
#include "prefmdl.h"

StorageModule *parseObj;
extern FILE *yyin;
extern void yyrestart(FILE *input_file);
extern PrefModule *prefs;

/*!
  \fn void StorageModule::push(QWidget *val)

  This member function is used to push widgets on the widget stack. This is
  neccesary for correct processing of recursive widget definitions.
*/

/*!
  \fn QWidget *StorageModule::pop()

  This member function is used to pop widgets off the widget stack. This is
  neccesary for correct processing of recursive widget definitions.
*/

/*!
  Create a new StorageModule.
*/

StorageModule::StorageModule(QObject *parent, const char *name)
: QObject(parent, name)
{
  level = 0;
  lineNumber = 1;
  sp = 0;
}

/*!
  Export the information contained in the WidgetInfo structure to C++.
  Constructs a header file called \a hname and an implementation file called
  \a iname in the directory \a dir.
*/

void StorageModule::export(WidgetInfo *info, const char *basename)
{
  QWidget *target = info->widget;
  QString base = basename;
  QString headerName = prefs->exportDir() + QString("/") + base;
  QString implName = headerName;
  headerName = headerName + prefs->headerExt();
  implName = implName + prefs->implExt();

  if(QFile::exists(headerName) || QFile::exists(implName))
  {
    int ret = QMessageBox::warning(0, "Ebuilder"
                                   , "Overwrite existing files?"
                                   , "&Yes", "&No", 0, 0, 1);
    if(ret)
      return;
  }

  // Export the header file
  ofstream fout;
  fout.open(headerName);
  if(! fout.good())
  {
    QMessageBox::critical(0, "Ebuilder", "Unable to open export file");
    return;
  }

  // Generate an #ifndef header
  fout << "#ifndef " << base << "_included\n";
  fout << "#define " << base << "_included\n";
  fout << endl;

  // Check which header files are needed
  int flags[14];
  for(int i=0; i<14; i++)
    flags[i] = 0;

  QObjectList *childList = target->queryList("QWidget");
  QObjectListIt objIt(*childList);
  while(objIt.current())
  {
    QString cname = ((QWidget *)objIt.current())->className();
    if(cname == "QCheckBox")
      flags[0]= 1;
    else if(cname == "QPushButton")
      flags[1]= 1;
    else if(cname == "QRadioButton")
      flags[2]= 1;
    else if(cname == "QComboBox")
      flags[3]= 1;
    else if(cname == "QFrame")
      flags[4]= 1;
    else if(cname == "QGroupBox")
      flags[5]= 1;
    else if(cname == "QButtonGroup")
      flags[6]= 1;
    else if(cname == "QLabel")
      flags[7]= 1;
    else if(cname == "QMenuBar")
      flags[8]= 1;
    else if(cname == "QListBox")
      flags[9]= 1;
    else if(cname == "QMultiLineEdit")
      flags[10]= 1;
    else if(cname == "QLineEdit")
      flags[11]= 1;
    else if(cname == "QScrollBar")
      flags[12]= 1;
    else if(cname == "QSlider")
      flags[13]= 1;
    ++objIt;
  }
  //delete childList;
  // Don't forget to delete the childList!!

  // Include the correct header file for the baseclass
  fout << "#include <" << info->baseHeader << ">\n";

  // Include other necessary header files
  if(flags[0])
    fout << "#include <qchkbox.h>\n";
  if(flags[1])
    fout << "#include <qpushbt.h>\n";
  if(flags[2])
    fout << "#include <qradiobt.h>\n";
  if(flags[3])
    fout << "#include <qcombo.h>\n";
  if(flags[4])
    fout << "#include <qframe.h>\n";
  if(flags[5])
    fout << "#include <qgrpbox.h>\n";
  if(flags[6])
    fout << "#include <qbttngrp.h>\n";
  if(flags[7])
    fout << "#include <qlabel.h>\n";
  if(flags[8])
    fout << "#include <qmenubar.h>\n";
  if(flags[9])
    fout << "#include <qlistbox.h>\n";
  if(flags[10])
    fout << "#include <qmlined.h>\n";
  if(flags[11])
    fout << "#include <qlined.h>\n";
  if(flags[12])
    fout << "#include <qscrbar.h>\n";
  if(flags[13])
    fout << "#include <qslider.h>\n";
  fout << endl;

  // Export extra header files
  QListIterator<IncludeFile> incIt(info->incList);
  for(; incIt.current(); ++incIt)
    fout << "#include <" << incIt.current()->name << ">\n";

  fout << endl;

  // Export the class declaration
  fout << "class " << info->name << ": public ";
  fout << info->baseClass << "\n";
  fout << "{\n";
  fout << "  Q_OBJECT\n";
  fout << "  public:\n";
  fout << "    " << info->name << "(" << info->arguments << ");\n";
  fout << "    ~" << info->name << "();\n";
  
  // Export all member functions found in the widget info structure
  // except constructor and destructor

  // Get the public member functions
  QListIterator<MemberFunction> memfIt(info->funcList);
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "public" && curr->extra == "")
    {
      fout << "    " << curr->type << " " << curr->name << "("
           << curr->arglist << ");\n";
    }
  }

  // A flag to see if members of a certain type are present
  int first;

  // Now export the public slots
  memfIt.toFirst();
  first = 1;
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "public" && curr->extra == "slot")
    {
      if(first)
      {
        first = 0;
        fout << "  public slots:\n";
      }
      fout << "    " << curr->type << " " << curr->name << "("
           << curr->arglist << ");\n";
    }
  }

  // Export the private slots
  memfIt.toFirst();
  first = 1;
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "private" && curr->extra == "slot")
    {
      if(first)
      {
        first = 0;
        fout << "  private slots:\n";
      }
      fout << "    " << curr->type << " " << curr->name << "("
           << curr->arglist << ");\n";
    }
  }

  // Next are the signals
  memfIt.toFirst();
  first = 1;
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->extra == "signal")
    {
      if(first)
      {
        first = 0;
        fout << "  signals:\n";
      }
      fout << "    " << curr->type << " " << curr->name << "("
           << curr->arglist << ");\n";
    }
  }

  // Finaly export the private member functions
  memfIt.toFirst();
  first = 1;
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "private" && curr->extra == "")
    {
      if(first)
      {
        first = 0;
        fout << "  " << curr->access << ":\n";
      }
      fout << "    " << curr->type << " " << curr->name << "("
           << curr->arglist << ");\n";
    }
  }

  // Export the member variables
  QListIterator<MemberVariable> memvIt(info->varList);

  const char *atype[] = {"public", "protected", "private"};
  for(int j=0; j<3; j++)
  {
    memvIt.toFirst();
    first = 1;
    for(; memvIt.current(); ++memvIt)
    {
      MemberVariable *curr = memvIt.current();
      if(curr->access == atype[j])
      {
        if(first)
        {
          first = 0;
          fout << "  " << curr->access << ":\n";
        }
        fout << "    " << curr->type << " " << curr->name << ";\n";
      }
    }
  }

  // Restart the child iteration to get all private members
  objIt.toFirst();
  fout << "  private:\n";
  while(objIt.current())
  {
    QWidget *curr = (QWidget *)objIt.current();
    QString cname = curr->className();
    QString ptype = curr->parent()->className();
    if(cname != "SizeSlab" && ptype != "QListBox" && ptype != "QMultiLineEdit")
      fout << "    " << curr->className() << " *" << curr->name() << ";\n";
    ++objIt;
  }
  fout << "};\n";
  //delete list;
  // Don't forget to delete the list!

  // Close the #ifndef header
  fout << endl;
  fout << "#endif\n";
  fout.close();

  // Export implementation file
  fout.open(implName);
  if(! fout.good())
  {
    QMessageBox::critical(0, "Ebuilder", "Unable to open file");
    return;
  }

  // Include the header file
  fout << "#include \"" << base << prefs->headerExt() << "\"\n\n";

  // Export the constructor implementation
  fout << info->name << "::" << info->name << "(" << info->arguments << ")\n";
  fout << info->baseInitializer << "\n";
  fout << "{\n";

  // Some more or less global variables needed to set widget properties
  fout << "  QFont fnt;\n";
  fout << "  QColor fg, bg, light, dark, mid, text, base;\n";
  fout << "  QColorGroup normal, disabled, active;\n";
  fout << "  QPalette pal;\n";
  fout << "\n";

  // Export the properties of the main window
  exportWidgetProps(target, fout);

  // Restart the child iteration to export the subwidgets
  objIt.toFirst();

  while(objIt.current())
  {
    QWidget *curr = (QWidget *)objIt.current();
    QString cname = curr->className();
    QString ptype = curr->parent()->className();

    if(cname != "SizeSlab" && ptype != "QListBox" && ptype != "QMultiLineEdit")
    {
      // Widget construction
      QString pname = curr->parent()->name();
      fout << "  " << curr->name() << " = new " << curr->className() << "(";

      if(pname == target->name())
        fout << "this);\n";
      else
        fout << curr->parent()->name() << ");\n";
      exportWidgetProps(curr, fout);
    }
    ++objIt;
  }
  delete childList;

  // Export extra code added to the constructor by the user
  memfIt.toFirst();
  fout << endl;
  for(; memfIt.current(); ++memfIt)
  {
    if(memfIt.current()->extra == "ctor")
    {
      if(memfIt.current()->text)
        fout << memfIt.current()->text;
      break;
    }
  }

  fout << "}\n";

  // Export the destructor, even though we don't need one. Perhaps the user
  // has entered extra coded
  memfIt.toFirst();
  for(; memfIt.current(); ++memfIt)
  {
    if(memfIt.current()->extra == "dtor")
    {
      fout << "\n" << target->name() << "::~" << target->name() << "()\n";
      fout << "{\n";
      if(memfIt.current()->text)
        fout << memfIt.current()->text;
      fout << "}\n";
      break;
    }
  }

  // Now export the implementation of the member functions
  memfIt.toFirst();
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "public" && curr->extra == "")
    {
      fout << "\n" << curr->type<< " " << target->name() << "::"
           << curr->name << "(" << curr->arglist << ")\n";
      fout << "{\n";
      fout << curr->text;
      fout << "}\n";
    }
  }
  memfIt.toFirst();
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "public" && curr->extra == "slot")
    {
      fout << "\n" << curr->type<< " " << target->name() << "::"
           << curr->name << "(" << curr->arglist << ")\n";
      fout << "{\n";
      fout << curr->text;
      fout << "}\n";
    }
  }
  memfIt.toFirst();
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "private" && curr->extra == "slot")
    {
      fout << "\n" << curr->type<< " " << target->name() << "::"
           << curr->name << "(" << curr->arglist << ")\n";
      fout << "{\n";
      fout << curr->text;
      fout << "}\n";
    }
  }
  memfIt.toFirst();
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    if(curr->access == "private" && curr->extra == "")
    {
      fout << "\n" << curr->type<< " " << target->name() << "::"
           << curr->name << "(" << curr->arglist << ")\n";
      fout << "{\n";
      fout << curr->text;
      fout << "}\n";
    }
  }

  fout.close();
}

/*!
  Exports the properties of the widget /a target to the file stream /a fout.
  All properties are checked to see if they correspond to the default values
  for the current application style. If a match is found, the property is
  not exported.
*/

void StorageModule::exportWidgetProps(QWidget *target, ofstream &fout)
{
  // Handle top level widgets in the correct way
  QString targetName;
  if(target->parent())
    targetName = target->name();
  else
    targetName = "this";

  // Export properties common to all widgets
  if(targetName == "this")
    fout << "  this->resize(" << target->width() << ", " << target->height()
         << ");\n";
  else
    fout << "  " << targetName << "->setGeometry(" << target->x() << ", "
         << target->y() << ", " << target->width() << ", " 
         << target->height() << ");\n";
  if(target->minimumSize() != QSize(0, 0))
  {
    fout << "  " << targetName << "->setMinimumSize("
         << target->minimumSize().width() << ", "
         << target->minimumSize().height() << ");\n";
  }
  if(target->maximumSize() != QSize(32767, 32767))
  {
    fout << "  " << targetName << "->setMaximumSize("
         << target->maximumSize().width() << ", "
         << target->maximumSize().height() << ");\n";
  }

  if(checkProperty(target, "Font"))
  {
    QFont fnt = target->font();
    fout << "  fnt = QFont(\"" << fnt.family() << "\", " << fnt.pointSize()
         << ", " << fnt.weight() << ");\n";
    fout << "  fnt.setItalic((bool)" << (int)fnt.italic() << ");\n";
    fout << "  fnt.setUnderline((bool)" << (int)fnt.underline() << ");\n";
    fout << "  fnt.setStrikeOut((bool)" << (int)fnt.strikeOut() << ");\n";
    fout << "  fnt.setFixedPitch((bool)" << (int)fnt.fixedPitch() << ");\n";
    fout << "  " << targetName << "->setFont(fnt);\n";
  }

  if(checkProperty(target, "Palette"))
  {
    QColor c; 

    c = target->palette().normal().foreground();
    fout << "  fg.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().normal().background();
    fout << "  bg.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().normal().light();
    fout << "  light.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().normal().dark();
    fout << "  dark.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().normal().mid();
    fout << "  mid.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().normal().text();
    fout << "  text.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().normal().base();
    fout << "  base.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    fout << "  normal = QColorGroup(fg, bg, light, dark, mid, text, base);\n";
  
    c = target->palette().disabled().foreground();
    fout << "  fg.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().disabled().background();
    fout << "  bg.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().disabled().light();
    fout << "  light.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().disabled().dark();
    fout << "  dark.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().disabled().mid();
    fout << "  mid.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().disabled().text();
    fout << "  text.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().disabled().base();
    fout << "  base.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    fout << "  disabled = QColorGroup(fg, bg, light, dark, mid, text, base);\n";
  
    c = target->palette().active().foreground();
    fout << "  fg.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().active().background();
    fout << "  bg.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().active().light();
    fout << "  light.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().active().dark();
    fout << "  dark.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().active().mid();
    fout << "  mid.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().active().text();
    fout << "  text.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    c = target->palette().active().base();
    fout << "  base.setRgb(" << c.red() << ", " << c.green() << ", "
         << c.blue() << ");\n";
    fout << "  active = QColorGroup(fg, bg, light, dark, mid, text, base);\n";
  
    fout << "  pal = QPalette(normal, disabled, active);\n";
    fout << "  " << targetName << "->setPalette(pal);\n";
  }

  if(checkProperty(target, "GUIStyle"))
  {
    fout << "  " << targetName << "->setStyle((GUIStyle)"
         << (int)target->style() << ");\n";
  }

  if(checkProperty(target, "FocusPolicy"))
  {
    fout << "  " << targetName << "->setFocusPolicy((QWidget::FocusPolicy)"
         << (int)target->focusPolicy() << ");\n";
  }

  if(checkProperty(target, "MouseTracking"))
  {
    fout << "  " << targetName << "->setMouseTracking((bool)"
         << (int)target->hasMouseTracking() << ");\n";
  }

  if(checkProperty(target, "Enable"))
  {
    fout << "  " << targetName << "->setEnabled((bool)"
         << (int)target->isEnabled() << ");\n";
  }

  if(checkProperty(target, "Caption"))
  {
    fout << "  " << targetName << "->setCaption(\""
         << target->caption() << "\");\n";
  }

  // Export widget specific stuff
  if(target->inherits("QButton"))
  {
    // Dump button properties
    QButton *tmp = (QButton *)target;
    if(checkProperty(target, "Text"))
    {
      QString tmpStr(tmp->text());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "  " << targetName << "->setText(\"" << tmpStr << "\");\n";
    }
    if(checkProperty(target, "AutoResize"))
    {
      fout << "  " << targetName << "->setAutoResize((bool)"
           << (int)tmp->autoResize() << ");\n";
    }
    if(checkProperty(target, "AutoRepeat"))
    {
      fout << "  " << targetName << "->setAutoRepeat((bool)"
           << (int)tmp->autoRepeat() << ");\n";
    }
  }
  else if(target->inherits("QFrame"))
  {
    // Dump frame properties
    QFrame *tmp = (QFrame *)target;
    if(checkProperty(target, "FrameStyle"))
    {
      fout << "  " << targetName << "->setFrameStyle("
           << (int)tmp->frameStyle() << ");\n";
    }
    if(checkProperty(target, "LineWidth"))
    {
      fout << "  " << targetName << "->setLineWidth("
           << (int)tmp->lineWidth() << ");\n";
    }
    if(checkProperty(target, "MidLineWidth"))
    {
      fout << "  " << targetName << "->setMidLineWidth("
           << (int)tmp->midLineWidth() << ");\n";
    }
  }
  else if(target->inherits("QRangeControl"))
  {
    // Dump rangecontrol properties
    QRangeControl *tmp = (QRangeControl *)target;
    if(checkProperty(target, "Value"))
    {
      fout << "  " << targetName << "->setValue(" << tmp->value() << ");\n";
    }
    if(checkProperty(target, "Range"))
    {
      fout << "  " << targetName << "->setRange(" << tmp->minValue() << ", "
           << tmp->maxValue() << ");\n";
    }
    if(checkProperty(target, "Steps"))
    {
      fout << "  " << targetName << "->setSteps(" << tmp->lineStep() << ", "
           << tmp->pageStep() << ");\n";
    }
  }

  // Dump widget specific stuff
  QString wtype = target->className();

  if(wtype == "QCheckBox")
  {
    QCheckBox *tmp = (QCheckBox *)target;
    if(checkProperty(target, "Checked"))
    {
      fout << "  " << targetName << "->setChecked((bool)" 
           << (int)tmp->isChecked() << ");\n";
    }
  }
  else if(wtype == "QPushButton")
  {
    QPushButton *tmp = (QPushButton *)target;
    if(checkProperty(target, "ToggleButton"))
    {
      fout << "  " << targetName << "->setToggleButton((bool)"
           << (int)tmp->isToggleButton() << ");\n";
    }
    if(checkProperty(target, "Default"))
    {
      fout << "  " << targetName << "->setDefault((bool)"
           << (int)tmp->isDefault() << ");\n";
    }
    if(checkProperty(target, "AutoDefault"))
    {
      fout << "  " << targetName << "->setAutoDefault((bool)"
           << (int)tmp->autoDefault() << ");\n";
    }
  }
  else if(wtype == "QRadioButton")
  {
    QRadioButton *tmp = (QRadioButton *)target;
    if(checkProperty(target, "Checked"))
    {
      fout << "  " << targetName << "->setChecked((bool)" 
           << (int)tmp->isChecked() << ");\n";
    }
  }
  else if(wtype == "QComboBox")
  {
    QComboBox *tmp = (QComboBox *)target;
    for(int i=0; i<tmp->count(); i++)
      fout << "  " << targetName << "->insertItem(\"" << tmp->text(i) 
           << "\");\n";
    if(checkProperty(target, "AutoResize"))
    {
      fout << "  " << targetName << "->setAutoResize((bool)"
           << (int)tmp->autoResize() << ");\n";
    }
    if(checkProperty(target, "SizeLimit"))
    {
      fout << "  " << targetName << "->setSizeLimit(" << tmp->sizeLimit()
           << ");\n";
    }
  }
  else if(wtype == "QGroupBox")
  {
    QGroupBox *tmp = (QGroupBox *)target;
    if(checkProperty(target, "Title"))
    {
      QString tmpStr(tmp->title());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "  " << targetName << "->setTitle(\"" << tmpStr << "\");\n";
    }
    if(checkProperty(target, "Alignment"))
    {
      fout << "  " << targetName << "->setAlignment(" << (int)tmp->alignment()
           << ");\n";
    }
  }
  else if(wtype == "QButtonGroup")
  {
    QButtonGroup *tmp = (QButtonGroup *)target;
    if(checkProperty(target, "Title"))
    {
      QString tmpStr(tmp->title());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "  " << targetName << "->setTitle(\"" << tmpStr << "\");\n";
    }
    if(checkProperty(target, "Alignment"))
    {
      fout << "  " << targetName << "->setAlignment(" << (int)tmp->alignment()
           << ");\n";
    }
    if(checkProperty(target, "Exclusive"))
    {
      fout << "  " << targetName << "->setExclusive((bool)"
           << (int)tmp->isExclusive() << ");\n";
    }
  }
  else if(wtype == "QListBox")
  {
    QListBox *tmp = (QListBox *)target;

    for(int i=0; i<(int)tmp->count(); i++)
    {
      fout << "  " << targetName << "->insertItem(\"" << tmp->text(i)
           << "\");\n";
    }
    if(checkProperty(target, "DragSelect"))
    {
      fout << "  " << targetName << "->setDragSelect((bool)"
           << (int)tmp->dragSelect() << ");\n";
    }
    if(checkProperty(target, "AutoScroll"))
    {
      fout << "  " << targetName << "->setAutoScroll((bool)"
           << (int)tmp->autoScroll() << ");\n";
    }
    if(checkProperty(target, "AutoScrollBar"))
    {
      fout << "  " << targetName << "->setAutoScrollBar((bool)"
           << (int)tmp->autoScrollBar() << ");\n";
    }
    if(checkProperty(target, "SmoothScroll"))
    {
      fout << "  " << targetName << "->setSmoothScrolling((bool)"
           << (int)tmp->smoothScrolling() << ");\n";
    }
  }
  else if(wtype == "QLineEdit")
  {
    QLineEdit *tmp = (QLineEdit *)target;
    if(checkProperty(target, "Text"))
    {
      fout << "  " << targetName << "->setText(\"" << tmp->text() << "\");\n";
    }
    if(checkProperty(target, "HasFrame"))
    {
      fout << "  " << targetName << "->setFrame((bool)" << (int)tmp->frame()
           << ");\n";
    }
    if(checkProperty(target, "EchoMode"))
    {
      fout << "  " << targetName << "->setEchoMode((QLineEdit::EchoMode)"
           << (int)tmp->echoMode() << ");\n";
    }
  }
  else if(wtype == "QMultiLineEdit")
  {
    QMultiLineEdit *tmp = (QMultiLineEdit *)target;

    for(int i=0; i<tmp->numLines(); i++)
      fout << "  " << targetName << "->insertLine(\"" << tmp->textLine(i)
           << "\");\n";
    if(checkProperty(target, "ReadOnly"))
    {
      fout << "  " << targetName << "->setReadOnly((bool)"
           << (int)tmp->isReadOnly() << ");\n";
    }
    if(checkProperty(target, "OverwriteMode"))
    {
      fout << "  " << targetName << "->setOverwriteMode((bool)"
           << (int)tmp->isOverwriteMode() << ");\n";
    }
  }
  else if(wtype == "QLabel")
  {
    QLabel *tmp = (QLabel *)target;
    if(checkProperty(target, "Text"))
    {
      QString tmpStr(tmp->text());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "  " << targetName << "->setText(\"" << tmpStr << "\");\n";
    }
    if(checkProperty(target, "Margin"))
    {
      fout << "  " << targetName << "->setMargin(" << tmp->margin() << ");\n";
    }
    if(checkProperty(target, "AutoResize"))
    {
      fout << "  " << targetName << "->setAutoResize((bool)"
           << (int)tmp->autoResize() << ");\n";
    }
    if(checkProperty(target, "Alignment"))
    {
      fout << "  " << targetName << "->setAlignment(" << tmp->alignment()
           << ");\n";
    }
  }
  else if(wtype == "QScrollBar")
  {
    QScrollBar *tmp = (QScrollBar *)target;
    if(checkProperty(target, "Value"))
    {
      fout << "  " << targetName << "->setValue(" << tmp->value() << ");\n";
    }
    if(checkProperty(target, "Range"))
    {
      fout << "  " << targetName << "->setRange(" << tmp->minValue() << ", "
           << tmp->maxValue() << ");\n";
    }
    if(checkProperty(target, "Steps"))
    {
      fout << "  " << targetName << "->setSteps(" << tmp->lineStep() << ", "
           << tmp->pageStep() << ");\n";
    }
    if(checkProperty(target, "Orientation"))
    {
      fout << "  " 
           << targetName << "->setOrientation((QScrollBar::Orientation)"
           << (int)tmp->orientation() << ");\n";
    }
    if(checkProperty(target, "Tracking"))
    {
      fout << "  " << targetName << "->setTracking((bool)"
           << (int)tmp->tracking() << ");\n";
    }
  }
  else if(wtype == "QSlider")
  {
    QSlider *tmp = (QSlider *)target;
    if(checkProperty(target, "Value"))
    {
      fout << "  " << targetName << "->setValue(" << tmp->value() << ");\n";
    }
    if(checkProperty(target, "Range"))
    {
      fout << "  " << targetName << "->setRange(" << tmp->minValue() << ", "
           << tmp->maxValue() << ");\n";
    }
    if(checkProperty(target, "Steps"))
    {
      fout << "  " << targetName << "->setSteps(" << tmp->lineStep() << ", "
           << tmp->pageStep() << ");\n";
    }
    if(checkProperty(target, "Orientation"))
    {
      fout << "  " << targetName << "->setOrientation((QSlider::Orientation)"
           << (int)tmp->orientation() << ");\n";
    }
    if(checkProperty(target, "Tracking"))
    {
      fout << "  " << targetName << "->setTracking((bool)"
           << (int)tmp->tracking() << ");\n";
    }
    if(checkProperty(target, "TickMarks"))
    {
      fout << "  " << targetName << "->setTickmarks((QSlider::TickSetting)"
           << (int)tmp->tickmarks() << ");\n";
    }
    if(checkProperty(target, "TickInterval"))
    {
      fout << "  " << targetName << "->setTickInterval("
           << tmp->tickInterval() << ");\n";
    }
  }
}

/*!
  Read a widget definition from file \a fname. This member function makes
  use of an external lexical analyser, generated with flex.
*/

WidgetInfo *StorageModule::read(const char *fname)
{
  FILE *fp = fopen(fname, "r");
  if(! fp)
  {
    QMessageBox::critical(0, "Ebuilder", "Unable to open file");
    return (WidgetInfo *)0;
  }

  // Reset counters
  lineNumber = 1;
  level = 0;
  sp = 0;

  // Set attributes used in parse.y
  parseObj = this;
  yyrestart(fp);

  // Create a new Widgetinfo structure
  WidgetInfo *ret = new WidgetInfo;
  if(!ret)
  {
    QMessageBox::critical(0, "Ebuilder", "Allocation failed");
    return (WidgetInfo *)0;
  }

  // Set parser attributes
  enum ParserState {Default, InWidget, InMemFun, InMemVar, InIncl};
  int error = 0;
  ParserState state = Default;

  // Objects created during parsing
  QWidget *curr=0;
  MemberFunction *mfEntry = 0;
  MemberVariable *mvEntry = 0;
  IncludeFile *incEntry = 0;

  // Start parsing
  int retVal;
  do
  {
    retVal = yylex();
    if(retVal == (int)WidgetStart)
    {
      //cerr << "Creating new Widget\n";
      QWidget *tmp;
      if(curr == 0)
      {
        ret->baseClass = type;
        ret->name = name;
        if(type == "QWidget")
        {
          tmp = addWidget(curr, "QWidget", name);
          ret->baseSubstitute = "QWidget";
	}
	else //(type == "QDialog")
        {
	  tmp = addWidget(curr, "QDialog", name);
          ret->baseSubstitute = "QDialog";
	}
	ret->widget = tmp;
      }
      else
      {
        tmp = addWidget(curr, type, name);
      }
      push(curr);
      curr = tmp;
      state = InWidget;
    }
    else if(retVal == (int)MemberFuncStart)
    {
      if(state == InWidget)
      {
        state = InMemFun;
        mfEntry = new MemberFunction;
      }
      else
      {
        QString buffer;
        buffer.setNum(lineNumber);
        buffer = "Line " + buffer + ": Member function outside widget";
        QMessageBox::critical(0, "Ebuilder", buffer);
        error = 1;
      }
    }
    else if(retVal == (int)MemberVarStart)
    {
      if(state == InWidget)
      {
        state = InMemVar;
        mvEntry = new MemberVariable;
      }
      else
      {
        QString buffer;
        buffer.setNum(lineNumber);
        buffer = "Line " + buffer + ": Member variable outside widget";
        QMessageBox::critical(0, "Ebuilder", buffer);
        error = 1;
      }
    }
    else if(retVal == (int)IncludeStart)
    {
      if(state == InWidget)
      {
        state = InIncl;
        incEntry = new IncludeFile;
      }
      else
      {
        QString buffer;
        buffer.setNum(lineNumber);
        buffer = "Line " + buffer + ": Include outside widget";
        QMessageBox::critical(0, "Ebuilder", buffer);
        error = 1;
      }
    }
    else if(retVal == (int)PropertyReady)
    {
      if(state == InWidget)
      {
        if(key == "Arguments")
          ret->arguments = value;
        else if(key == "BaseClass")
          ret->baseClass = value;
        else if(key == "BaseInit")
          ret->baseInitializer = value;
        else if(key == "BaseHeader")
          ret->baseHeader = value;
        else if(key == "Modal")
          ret->modal = (bool)atoi(value);
        else
        {
          if(! setWidgetProperty(curr, key, value))
          {
            QString buffer;
            buffer.setNum(lineNumber);
            buffer = "Line " + buffer + ": Error in widget property";
            QMessageBox::critical(0, "Ebuilder", buffer);
            error = 1;
          }
        }
      }
      else if(state == InMemFun)
      {
        if(! setMemberFuncProperty(mfEntry, key, value))
        {
          QString buffer;
          buffer.setNum(lineNumber);
          buffer = "Line " + buffer + ": Error in member function property";
          QMessageBox::critical(0, "Ebuilder", buffer);
          error = 1;
        }
      }
      else if(state == InMemVar)
      {
        if(! setMemberVarProperty(mvEntry, key, value))
        {
          QString buffer;
          buffer.setNum(lineNumber);
          buffer = "Line " + buffer + ": Error in member variable property";
          QMessageBox::critical(0, "Ebuilder", buffer);
          error = 1;
        }
      }
      else if(state == InIncl)
      {
        if(! setIncludeProperty(incEntry, key, value))
        {
          QString buffer;
          buffer.setNum(lineNumber);
          buffer = "Line " + buffer + ": Error in include property";
          QMessageBox::critical(0, "Ebuilder", buffer);
          error = 1;
        }
      }
      else
      {
        QString buffer;
        buffer.setNum(lineNumber);
        buffer = "Line " + buffer + ": Property outside widget";
        QMessageBox::critical(0, "Ebuilder", buffer);
        error = 1;
      }
    }
    else if(retVal == (int)OpenBracket)
    {
      if(state != Default)
        level++;
      else
      {
        QString buffer;
        buffer.setNum(lineNumber);
        buffer = "Line " + buffer + ": Unexpected open bracket";
        QMessageBox::critical(0, "Ebuilder", buffer);
        error = 1;
      }
    }
    else if(retVal == (int)CloseBracket)
    {
      if(state == Default)
      {
        QString buffer;
        buffer.setNum(lineNumber);
        buffer = "Line " + buffer + ": Unexpected close bracket";
        QMessageBox::critical(0, "Ebuilder", buffer);
        error = 1;
      }
      else if(state == InWidget)
      {
        //cerr << "Closing widget\n";
        level--;
        if(level == 0)
        {
          //cerr << "Widget definition complete\n";
          pop();
          state = Default;
          break;
        }
        else if(level < 0)
        {
          QString buffer;
          buffer.setNum(lineNumber);
          buffer = "Line " + buffer + ": Unexpected close bracket";
          QMessageBox::critical(0, "Ebuilder", buffer);
          error = 1;
        }
        else
          curr = pop();
      }
      else if(state == InMemFun)
      {
        ret->funcList.append(mfEntry);
        level--;
        state = InWidget;
      }
      else if(state == InMemVar)
      {
        ret->varList.append(mvEntry);
        level--;
        state = InWidget;
      }
      else if(state == InIncl)
      {
        ret->incList.append(incEntry);
        level--;
        state = InWidget;
      }
    }
    else if(retVal == EndOfInput)
    {
      QString buffer;
      buffer.setNum(lineNumber);
      buffer = "Line " + buffer + ": Unexpected end of file";
      QMessageBox::critical(0, "Ebuilder", buffer);
      error = 1;
    }
    else
    {
      QString buffer;
      buffer.setNum(lineNumber);
      buffer = "Line " + buffer + ": Unexpected symbol";
      QMessageBox::critical(0, "Ebuilder", buffer);
      error = 1;
    }
  } while(error == 0);

  fclose(fp);

  if(error == 1)
  {
    delete ret;
    return (WidgetInfo *)0;
  }

  return ret;
}

/*!
  Write a widget definition to file \a fname.
  \sa StorageModule::writeWidgetProps
      StorageModule::writeMemberFuncs
      StorageModule::writeMemberVars
      StorageModule::writeIncludes
*/

void StorageModule::write(WidgetInfo *info, const char *fname)
{
  ofstream fout;
  fout.open(fname);
  if(! fout.good())
  {
    QMessageBox::critical(0, "Ebuilder", "Unable to open file");
    return;
  }

  // First write the top level widget. Some special precautions are needed
  // there. The info->widget member does not necessarily hold a widget of
  // the correct type. The info->baseSubstitute is needed to get the correct
  // type.

  // Save main widget basetype
  fout << info->baseSubstitute;
  fout << "[" << info->name << "]";
  fout << "\n{\n";

  // Save baseclass information for the main widget
  fout << "Arguments {" << info->arguments << "}\n";
  fout << "BaseClass {" << info->baseClass << "}\n";
  fout << "BaseInit {" << info->baseInitializer << "}\n";
  fout << "BaseHeader {" << info->baseHeader << "}\n";
  fout << "Modal {" << (int)info->modal << "}\n";

  // Save the properties of the main window and all its subwidget
  // (writeWidgetProp is recursive)
  writeWidgetProps(info->widget, fout);

  // Write other widget information
  writeMemberFuncs(info->funcList, fout);
  writeMemberVars(info->varList, fout);
  writeIncludes(info->incList, fout);

  fout << "}\n";
  fout.close();
}

/*!
  Add a widget of type \a type to the widget pointed to by \a target. Give
  it name \a name. Used by StorageModule::read.
*/

QWidget *StorageModule::addWidget(QWidget *parent, const QString &type,
                                  const QString &name)
{
  //cerr << "Opening " << type << ": " << name <<endl;
  QWidget *ret;
  if(type == "QWidget")
    ret = new QWidget(parent, name);
  else if(type == "QDialog")
    ret = new QDialog(parent, name, FALSE);
  else if(type == "QCheckBox")
    ret = new QCheckBox(parent, name);
  else if(type == "QPushButton")
    ret = new QPushButton(parent, name);
  else if(type == "QRadioButton")
    ret = new QRadioButton(parent, name);
  else if(type == "QComboBox")
    ret = new QComboBox(parent, name);
  else if(type == "QFrame")
    ret = new QFrame(parent, name);
  else if(type == "QGroupBox")
    ret = new QGroupBox(parent, name);
  else if(type == "QButtonGroup")
    ret = new QButtonGroup(parent, name);
  else if(type == "QLineEdit")
    ret = new QLineEdit(parent, name);
  else if(type == "QMultiLineEdit")
    ret = new QMultiLineEdit(parent, name);
  else if(type == "QLabel")
    ret = new QLabel(parent, name);
  else if(type == "QScrollBar")
    ret = new QScrollBar(parent, name);
  else if(type == "QSlider")
    ret = new QSlider(parent, name);
  else if(type == "QListBox")
    ret = new QListBox(parent, name);
  else if(type == "QMenuBar")
    ret = new QMenuBar(parent, name);
  else
  {
    ret = 0;
    cerr << "*** Unkown widget type\n";
  }
  if(ret)
  {
    ret->show();
    ret->setStyle(prefs->defaultStyle());
  }

  return ret;
}

/*!
  Set the property of the widget \a target named \a key to \a val. This
  member checks if the properties are valid. Used by StorageModule::read.

  Return values:
    0 if any error occurs
    1 if all is well
*/

int StorageModule::setWidgetProperty(QWidget *target, const QString &key,
                                      const QString &val)
{
  //cerr << "Changing property: " << key << " = " << val << endl;
  if(!target)
    return 0;
  QString tclass = target->className();
  if(key == "Geometry")
  {
    int x, y, w, h, minw, minh, maxw, maxh;
    sscanf(val, "%d%d%d%d%d%d%d%d", &x, &y, &w, &h,
           &minw, &minh, &maxw, &maxh);
    target->setGeometry(x, y, w, h);
    target->setMinimumSize(QSize(minw, minh));
    target->setMaximumSize(QSize(maxw, maxh));
  }
  else if(key == "Font")
  {
    char family[256];
    int psize, weight;
    int it, ul, st, fx;
    sscanf(val, "%s%d%d%d%d%d%d", family, &psize, &weight,
           &it, &ul, &st, &fx);
    QFont fnt(family, psize, weight, (bool)it);
    fnt.setUnderline((bool)ul);
    fnt.setStrikeOut((bool)st);
    fnt.setFixedPitch((bool)fx);
    target->setFont(fnt);
  }
  else if(key == "Palette")
  {
    char *buffer = new char[val.length()];
    strcpy(buffer, val);

    int i=0;
    char *token = strtok(buffer, " ");

    QColorGroup normal, disabled, active;
    while(token)
    {
      QColor fg, bg, light, dark, mid, text, base;
      char *r, *g, *b;
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      fg.setRgb(atoi(r), atoi(g), atoi(b));
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      bg.setRgb(atoi(r), atoi(g), atoi(b));
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      light.setRgb(atoi(r), atoi(g), atoi(b));
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      dark.setRgb(atoi(r), atoi(g), atoi(b));
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      mid.setRgb(atoi(r), atoi(g), atoi(b));
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      text.setRgb(atoi(r), atoi(g), atoi(b));
      r = token; token = strtok(0, " ");
      g = token; token = strtok(0, " ");
      b = token; token = strtok(0, " ");
      base.setRgb(atoi(r), atoi(g), atoi(b));

      if(i == 0)
        normal = QColorGroup(fg, bg, light, dark, mid, text, base);
      else if(i == 1)
        disabled = QColorGroup(fg, bg, light, dark, mid, text, base);
      else if(i == 2)
        active = QColorGroup(fg, bg, light, dark, mid, text, base);

      i++;
    }
    delete buffer;
    target->setPalette(QPalette(normal, disabled, active));
  }
  else if(key == "GUIStyle")
  {
    target->setStyle((GUIStyle)atoi(val));
  }
  else if(key == "FocusPolicy")
  {
    target->setFocusPolicy((QWidget::FocusPolicy)atoi(val));
  }
  else if(key == "MouseTracking")
  {
    target->setMouseTracking((bool)atoi(val));
  }
  else if(key == "Enable")
  {
    target->setEnabled((bool)atoi(val));
  }
  else if(key == "Caption")
  {
    target->setCaption(val);
  }
  else if(key == "Default")
  {
    if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      tmp->setDefault((bool)atoi(val));
    }
  }
  else if(key == "AutoDefault")
  {
    if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      tmp->setAutoDefault((bool)atoi(val));
    }
  }
  else if(key == "AutoResize")
  {
    if(target->inherits("QButton"))
    {
      QButton *tmp = (QButton *)target;
      tmp->setAutoResize((bool)atoi(val));
    }
    else if(target->inherits("QComboBox"))
    {
      QComboBox *tmp = (QComboBox *)target;
      tmp->setAutoResize((bool)atoi(val));
    }
    else if(target->inherits("QLabel"))
    {
      QLabel *tmp = (QLabel *)target;
      tmp->setAutoResize((bool)atoi(val));
    }
  }
  else if(key == "AutoRepeat")
  {
    if(target->inherits("QButton"))
    {
      QButton *tmp = (QButton *)target;
      tmp->setAutoRepeat((bool)atoi(val));
    }
  }
  else if(key == "TextLine")
  {
    if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      tmp->insertLine(val);
    }
  }
  else if(key == "Text")
  {
    QString valStr(val);
    int idx = valStr.find("\\n");
    while(idx != -1)
    {
      valStr = valStr.left(idx) + '\n'
               + valStr.right(valStr.length()-idx-2);
      idx = valStr.find("\\n", idx);
    }
  
    if(target->inherits("QButton"))
    {
      QButton *tmp = (QButton *)target;
      tmp->setText(valStr);
    }
    else if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      tmp->setText(valStr);
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      tmp->setText(valStr);
    }
  }
  else if(key == "Checked")
  {
    if(tclass == "QCheckBox")
    {
      QCheckBox *tmp = (QCheckBox *)target;
      tmp->setChecked((bool)atoi(val));
    }
    else if(tclass == "QRadioButton")
    {
      QRadioButton *tmp = (QRadioButton *)target;
      tmp->setChecked((bool)atoi(val));
    }
  }
  else if(key == "ToggleButton")
  {
    if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      tmp->setToggleButton((bool)atoi(val));
    }
  }
  else if(key == "Item")
  {
    if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      tmp->insertItem(val);
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      tmp->insertItem(val);
    }
  }
  else if(key == "SizeLimit")
  {
    if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      tmp->setSizeLimit(atoi(val));
    }
  }
  else if(key == "FrameStyle")
  {
    if(target->inherits("QFrame"))
    {
      QFrame *tmp = (QFrame *)target;
      tmp->setFrameStyle(atoi(val));
    }
  }
  else if(key == "LineWidth")
  {
    if(target->inherits("Frame"))
    {
      QFrame *tmp = (QFrame *)target;
      tmp->setLineWidth(atoi(val));
    }
  }
  else if(key == "MidLineWidth")
  {
    if(target->inherits("QFrame"))
    {
      QFrame *tmp = (QFrame *)target;
      tmp->setMidLineWidth(atoi(val));
    }
  }
  else if(key == "Title")
  {
    QString valStr(val);
    int idx = valStr.find("\\n");
    while(idx != -1)
    {
      valStr = valStr.left(idx) + '\n'
               + valStr.right(valStr.length()-idx-2);
      idx = valStr.find("\\n", idx);
    }
    if(target->inherits("QGroupBox"))
    {
      QGroupBox *tmp = (QGroupBox *)target;
      tmp->setTitle(valStr);
    }
  }
  else if(key == "Alignment")
  {
    if(target->inherits("QGroupBox"))
    {
      QGroupBox *tmp = (QGroupBox *)target;
      tmp->setAlignment(atoi(val));
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      tmp->setAlignment(atoi(val));
    }
  }
  else if(key == "Exclusive")
  {
    if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      tmp->setExclusive((bool)atoi(val));
    }
  }
  else if(key == "HasFrame")
  {
    if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      tmp->setFrame((bool)atoi(val));
    }
  }
  else if(key == "EchoMode")
  {
    if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      tmp->setEchoMode((QLineEdit::EchoMode)atoi(val));
    }
  }
  else if(key == "ReadOnly")
  {
    if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      tmp->setReadOnly((bool)atoi(val));
    }
  }
  else if(key == "OverwriteMode")
  {
    if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      tmp->setOverwriteMode((bool)atoi(val));
    }
  }
  else if(key == "DragSelect")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      tmp->setDragSelect((bool)atoi(val));
    }
  }
  else if(key == "AutoScroll")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      tmp->setAutoScroll((bool)atoi(val));
    }
  }
  else if(key == "AutoScrollBar")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      tmp->setAutoScrollBar((bool)atoi(val));
    }
  }
  else if(key == "SmoothScroll")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      tmp->setSmoothScrolling((bool)atoi(val));
    }
  }
  else if(key == "Range")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      int min, max;
      sscanf(val, "%d%d", &min, &max);
      tmp->setRange(min, max);
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      int min, max;
      sscanf(val, "%d%d", &min, &max);
      tmp->setRange(min, max);
    }
  }
  else if(key == "Steps")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      int line, page;
      sscanf(val, "%d%d", &line, &page);
      tmp->setSteps(line, page);
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      int line, page;
      sscanf(val, "%d%d", &line, &page);
      tmp->setSteps(line, page);
    }
  }
  else if(key == "Value")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      tmp->setValue(atoi(val));
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      tmp->setValue(atoi(val));
    }
  }
  else if(key == "Orientation")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      tmp->setOrientation((QScrollBar::Orientation)atoi(val));
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      tmp->setOrientation((QSlider::Orientation)atoi(val));
    }
  }
  else if(key == "Tracking")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      tmp->setTracking((bool)atoi(val));
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      tmp->setTracking((bool)atoi(val));
    }
  }
  else if(key == "Tickmarks")
  {
    if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      tmp->setTickmarks((QSlider::TickSetting)atoi(val));
    }
  }
  else if(key == "TickInterval")
  {
    if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      tmp->setTickInterval(atoi(val));
    }
  }
  else if(key == "Margin")
  {
    if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      tmp->setMargin(atoi(val));
    }
  }
  else
  {
    cerr << "*** Unknown key : " << key << ", ignoring\n";
    return 0;
  }
  return 1; //Success
}


/*!
  Set property \a key of MemberFunction \a target to \a val. This member
  checks if the key is correct. Used by StorageModule::read.

  Return values:

    0 on any error
    1 if all is well
*/

int StorageModule::setMemberFuncProperty(MemberFunction *target
                                          , const QString &key
                                          , const QString &val)
{
  //cerr << "Changing property: " << key << " = " << val << endl;
  if(!target)
    return 0;
  if(key == "RetType")
    target->type = val;
  else if(key == "Name")
    target->name = val;
  else if(key == "ArgList")
    target->arglist = val;
  else if(key == "Access")
    target->access = val;
  else if(key == "Extra")
    target->extra = val;
  else if(key == "CodeLine")
    target->text = target->text + val + "\n";
  else
  {
    cerr << "*** Unknown key : " << key << ", ignoring\n";
    return 0;
  }
  return 1; //Success
}

/*!
  Set property \a key of MemberVariable \a target to \a val. This member
  checks if the key is correct. Used by StorageModule::read.

  Return values:

    0 on any error
    1 if all is well
*/

int StorageModule::setMemberVarProperty(MemberVariable *target
                                         , const QString &key
                                         , const QString &val)
{
  //cerr << "Changing property: " << key << " = " << val << endl;
  if(!target)
    return 0;
  if(key == "Type")
    target->type = val;
  else if(key == "Name")
    target->name = val;
  else if(key == "Access")
    target->access = val;
  else
  {
    cerr << "*** Unknown key : " << key << ", ignoring\n";
    return 0;
  }
  return 1; //Success
}

/*!
  Set property \a key of IncludeFile \a target to \a val. This member
  checks if the key is correct. Used by StorageModule::read.

  Return values:

    0 on any error
    1 if all is well
*/

int StorageModule::setIncludeProperty(IncludeFile *target
                                         , const QString &key
                                         , const QString &val)
{
  //cerr << "Changing property: " << key << " = " << val << endl;
  if(!target)
    return 0;
  if(key == "Name")
    target->name = val;
  else
  {
    cerr << "*** Unknown key : " << key << ", ignoring\n";
    return 0;
  }
  return 1; //Success
}

/*!
  Write the properties of the widget \a target to output file stream \a
  fout. This member calls itself recursively to write the properties of all
  subwidgets of \a target to disk. Used by StorageModule::write.
*/

void StorageModule::writeWidgetProps(QWidget *target, ofstream &fout)
{
  // Dump properties common to all widgets
  fout << "Geometry {"
       << target->x() << " " <<  target->y() << " "
       << target->width() << " " << target->height() << " "
       << target->minimumSize().width() << " "
       << target->minimumSize().height() << " "
       << target->maximumSize().width() << " "
       << target->maximumSize().height() << "}\n";

  if(checkProperty(target, "Font"))
  {
    const QFont &fnt = target->font();
    fout << "Font {" << fnt.family() << " " << fnt.pointSize() << " "
         << fnt.weight() << " " << (int)fnt.italic() << " "
         << (int)fnt.underline() << " " << (int)fnt.strikeOut() << " "
         << (int)fnt.fixedPitch() << "}\n";
  }

  if(checkProperty(target, "Palette"))
  {
    QColorGroup grp;
    QColor col;
    fout << "Palette {";
    grp = target->palette().normal();
    col = grp.foreground();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.background();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.light();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.dark();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.mid();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.text();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.base();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";

    grp = target->palette().disabled();
    col = grp.foreground();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.background();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.light();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.dark();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.mid();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.text();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.base();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";

    grp = target->palette().active();
    col = grp.foreground();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.background();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.light();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.dark();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.mid();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.text();
    fout << col.red() << " " << col.green() << " " << col.blue() << " ";
    col = grp.base();
    fout << col.red() << " " << col.green() << " " << col.blue();
    fout << "}\n";
  }

  if(checkProperty(target, "GUIStyle"))
    fout << "GUIStyle {" << (int)target->style() << "}\n";
  if(checkProperty(target, "FocusPolicy"))
    fout << "FocusPolicy {" << (int)target->focusPolicy() << "}\n";
  if(checkProperty(target, "MouseTracking"))
    fout << "MouseTracking {" << (int)target->hasMouseTracking() << "}\n";
  if(checkProperty(target, "Enable"))
    fout << "Enable {" << (int)target->isEnabled() << "}\n";
  if(checkProperty(target, "Caption"))
    fout << "Caption {" << target->caption()<< "}\n";

  // Dump widget specific stuff
  if(target->inherits("QButton"))
  {
    // Dump button properties
    QButton *tmp = (QButton *)target;
    if(checkProperty(target, "Text"))
    {
      QString tmpStr(tmp->text());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "Text {" <<  tmpStr << "}\n";
    }
    if(checkProperty(target, "AutoResize"))
      fout << "AutoResize {" << (int)tmp->autoResize() << "}\n";
    if(checkProperty(target, "AutoRepeat"))
      fout << "AutoRepeat {" << (int)tmp->autoRepeat() << "}\n";
  }
  else if(target->inherits("QFrame"))
  {
    // Dump frame properties
    QFrame *tmp = (QFrame *)target;
    if(checkProperty(target, "FrameStyle"))
      fout << "FrameStyle {" << (int)tmp->frameStyle() << "}\n";
    if(checkProperty(target, "LineWidth"))
      fout << "LineWidth {" << (int)tmp->lineWidth() << "}\n";
    if(checkProperty(target, "MidLineWidth"))
      fout << "MidLineWidth {" << (int)tmp->midLineWidth() << "}\n";
  }
  else if(target->inherits("QRangeControl"))
  {
    // Dump rangecontrol properties
    QRangeControl *tmp = (QRangeControl *)target;
    if(checkProperty(target, "Value"))
      fout << "Value {" <<  tmp->value() << "}\n";
    if(checkProperty(target, "Range"))
      fout << "Range {" << tmp->minValue() << " " << tmp->maxValue() << "}\n";
    if(checkProperty(target, "Steps"))
      fout << "Steps {" << tmp->lineStep()<< " " << tmp->pageStep() << "}\n";
  }

  // Dump widget specific stuff
  QString wtype = target->className();

  if(wtype == "QCheckBox")
  {
    QCheckBox *tmp = (QCheckBox *)target;
    if(checkProperty(target, "Checked"))
      fout << "Checked {" << (int)tmp->isChecked() << "}\n";
  }
  else if(wtype == "QPushButton")
  {
    QPushButton *tmp = (QPushButton *)target;
    if(checkProperty(target, "ToggleButton"))
      fout << "ToggleButton {" << (int)tmp->isToggleButton() << "}\n";
    if(checkProperty(target, "AutoDefault"))
      fout << "AutoDefault {" << (int)tmp->autoDefault() << "}\n";
    if(checkProperty(target, "Default"))
      fout << "Default {" << (int)tmp->isDefault() << "}\n";
  }
  else if(wtype == "QRadioButton")
  {
    QRadioButton *tmp = (QRadioButton *)target;
    if(checkProperty(target, "Checked"))
      fout << "Checked {" << (int)tmp->isChecked() << "}\n";
  }
  else if(wtype == "QComboBox")
  {
    QComboBox *tmp = (QComboBox *)target;
    for(int i=0; i<tmp->count(); i++)
      fout << "Item {" << tmp->text(i) << "}\n";
    if(checkProperty(target, "AutoResize"))
      fout << "AutoResize {" << (int)tmp->autoResize() << "}\n";
    if(checkProperty(target, "SizeLimit"))
      fout << "SizeLimit {" << tmp->sizeLimit() << "}\n";
  }
  else if(wtype == "QGroupBox")
  {
    QGroupBox *tmp = (QGroupBox *)target;
    if(checkProperty(target, "Title"))
    {
      QString tmpStr(tmp->title());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "Title {" <<  tmpStr << "}\n";
    }
    if(checkProperty(target, "Alignment"))
      fout << "Alignment {" << (int)tmp->alignment() << "}\n";
  }
  else if(wtype == "QButtonGroup")
  {
    QButtonGroup *tmp = (QButtonGroup *)target;
    if(checkProperty(target, "Title"))
    {
      QString tmpStr(tmp->title());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "Title {" << tmpStr << "}\n";
    }
    if(checkProperty(target, "Alignment"))
      fout << "Alignment {" << (int)tmp->alignment() << "}\n";
    if(checkProperty(target, "Exclusive"))
      fout << "Exclusive {" << (int)tmp->isExclusive() << "}\n";
  }
  else if(wtype == "QListBox")
  {
    QListBox *tmp = (QListBox *)target;
    for(int i=0; i<(int)tmp->count(); i++)
      fout << "Item {" << tmp->text(i) << "}\n";
    if(checkProperty(target, "DragSelect"))
      fout << "DragSelect {" << (int)tmp->dragSelect() << "}\n";
    if(checkProperty(target, "AutoScroll"))
      fout << "AutoScroll {" << (int)tmp->autoScroll() << "}\n";
    if(checkProperty(target, "AutoScrollBar"))
      fout << "AutoScrollBar {" << (int)tmp->autoScrollBar() << "}\n";
    if(checkProperty(target, "SmoothScroll"))
      fout << "SmoothScroll {" << (int)tmp->smoothScrolling() << "}\n";
  }
  else if(wtype == "QLineEdit")
  {
    QLineEdit *tmp = (QLineEdit *)target;
    if(checkProperty(target, "Text"))
      fout << "Text {" << tmp->text() << "}\n";
    if(checkProperty(target, "HasFrame"))
      fout << "HasFrame {" << (int)tmp->frame() << "}\n";
    if(checkProperty(target, "EchoMode"))
      fout << "EchoMode {" << (int)tmp->echoMode() << "}\n";
  }
  else if(wtype == "QMultiLineEdit")
  {
    QMultiLineEdit *tmp = (QMultiLineEdit *)target;
    for(int i=0; i<tmp->numLines(); i++)
      fout << "TextLine {" << tmp->textLine(i) << "}\n";

    if(checkProperty(target, "ReadOnly"))
      fout << "ReadOnly {" << (int)tmp->isReadOnly() << "}\n";
    if(checkProperty(target, "OverwriteMode"))
      fout << "OverwriteMode {" << (int)tmp->isOverwriteMode() << "}\n";
  }
  else if(wtype == "QLabel")
  {
    QLabel *tmp = (QLabel *)target;
    if(checkProperty(target, "Text"))
    {
      QString tmpStr(tmp->text());
      tmpStr.replace(QRegExp("\n"), "\\n");
      fout << "Text {" << tmpStr << "}\n";
    }
    if(checkProperty(target, "Margin"))
      fout << "Margin {" << tmp->margin() << "}\n";
    if(checkProperty(target, "Alignment"))
      fout << "Alignment {" << (int)tmp->alignment() << "}\n";
    if(checkProperty(target, "AutoResize"))
      fout << "AutoResize {" << (int)tmp->autoResize() << "}\n";
  }
  else if(wtype == "QScrollBar")
  {
    QScrollBar *tmp = (QScrollBar *)target;
    if(checkProperty(target, "Value"))
      fout << "Value {" <<  tmp->value() << "}\n";
    if(checkProperty(target, "Range"))
      fout << "Range {" << tmp->minValue() << " " << tmp->maxValue() << "}\n";
    if(checkProperty(target, "Steps"))
      fout << "Steps {" << tmp->lineStep()<< " " << tmp->pageStep() << "}\n";
    if(checkProperty(target, "Orientation"))
      fout << "Orientation {" << tmp->orientation() << "}\n";
    if(checkProperty(target, "Tracking"))
      fout << "Tracking {" << (int)tmp->tracking() << "}\n";
  }
  else if(wtype == "QSlider")
  {
    QSlider *tmp = (QSlider *)target;
    if(checkProperty(target, "Value"))
      fout << "Value {" <<  tmp->value() << "}\n";
    if(checkProperty(target, "Range"))
      fout << "Range {" << tmp->minValue() << " " << tmp->maxValue() << "}\n";
    if(checkProperty(target, "Steps"))
      fout << "Steps {" << tmp->lineStep()<< " " << tmp->pageStep() << "}\n";
    if(checkProperty(target, "Orientation"))
      fout << "Orientation {" << tmp->orientation() << "}\n";
    if(checkProperty(target, "Tracking"))
      fout << "Tracking {" << (int)tmp->tracking() << "}\n";
    if(checkProperty(target, "TickMarks"))
      fout << "Tickmarks {" << (int)tmp->tickmarks() << "}\n";
    if(checkProperty(target, "TickInterval"))
      fout << "TickInterval {" << tmp->tickInterval() << "}\n";
  }

  // Write all subwidgets, but prevent dumping of internal subwidgets such as
  // scrollbars used by ListBox and MultiLineEdit
  QObjectList *list = target->queryList("QWidget", 0, FALSE, FALSE);
  QObjectListIt it(*list);
  for(; it.current(); ++it)
  {
    QString pname = it.current()->parent()->className();
    QString cname = it.current()->className();
    if(pname != "QListBox" && pname != "QMultiLineEdit" && cname != "SizeSlab")
    {
      fout << cname;
      if(it.current()->name())
        fout << "[" << it.current()->name() << "]";
      fout << "\n{\n";
      writeWidgetProps((QWidget *)it.current(), fout);
      fout << "}\n";
    }
  }
  delete list;
}

/*!
  Writes all MemberFunction-s found in \a list to output file stream \a
  fout. Used by StorageModule::write.
*/

void StorageModule::writeMemberFuncs(const QList<MemberFunction> &list,
                                   ofstream &fout)
{
  QListIterator<MemberFunction> memfIt(list);
  for(; memfIt.current(); ++memfIt)
  {
    MemberFunction *curr = memfIt.current();
    fout << "MemberFunction\n";
    fout << "{\n";
    fout << "  RetType {" << curr->type << "}\n";
    fout << "  Name {" << curr->name << "}\n";
    fout << "  ArgList {" << curr->arglist << "}\n";
    fout << "  Access {" << curr->access << "}\n";
    fout << "  Extra {" << curr->extra << "}\n";

    if(curr->text.length())
    {
      int idx1 = 0, idx2 = -1;
      do
      {
        idx2 = curr->text.find('\n', idx1);
        if(idx2 > 0)
          fout << "  CodeLine {" << curr->text.mid(idx1, idx2-idx1) << "}\n";
        idx1 = idx2 + 1;
      }
      while(idx2 != -1);
    }

    fout << "}\n";
  }
}

/*!
  Writes all MemberVariables-s found in \a list to output file stream \a
  fout. Used by StorageModule::write.
*/

void StorageModule::writeMemberVars(const QList<MemberVariable> &list,
                                  ofstream &fout)
{
  QListIterator<MemberVariable> memvIt(list);
  for(; memvIt.current(); ++memvIt)
  {
    MemberVariable *curr = memvIt.current();
    fout << "MemberVariable\n";
    fout << "{\n";
    fout << "  Type {" << curr->type << "}\n";
    fout << "  Name {" << curr->name << "}\n";
    fout << "  Access {" << curr->access << "}\n";
    fout << "}\n";
  }
}

/*!
  Writes all IncludeFile-s found in \a list to output file stream \a
  fout. Used by StorageModule::write.
*/

void StorageModule::writeIncludes(const QList<IncludeFile> &list,
                                 ofstream &fout)
{
  QListIterator<IncludeFile> incIt(list);
  for(; incIt.current(); ++incIt)
  {
    fout << "Include\n";
    fout << "{\n";
    fout << "  Name {" << incIt.current()->name << "}\n";
    fout << "}\n";
  }
}

/*!
  Checks the property \a property of \a target agains the default value.

  Return values:
    0 if the properties are the same
    1 if the properties differ
*/

int StorageModule::checkProperty(QWidget *target, const char *property)
{
  QString propStr(property);
  QString tclass = target->className();
  //cerr << "checking property: " << property << "  of type: " << tclass
  //     << endl;
  if(propStr == "Alignment")
  {
    if(target->inherits("QGroupBox"))
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->alignment() == cmp.alignment())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->alignment() == cmp.alignment())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "AutoDefault")
  {
    if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoDefault() == cmp.autoDefault())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "AutoRepeat")
  {
    if(target->inherits("QButton"))
    {
      QButton *tmp = (QButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoRepeat() == cmp.autoRepeat())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "AutoResize")
  {
    if(target->inherits("QButton"))
    {
      QButton *tmp = (QButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoResize() == cmp.autoResize())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      QComboBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoResize() == cmp.autoResize())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoResize() == cmp.autoResize())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "AutoScroll")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoScroll() == cmp.autoScroll())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "AutoScrollBar")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->autoScrollBar() == cmp.autoScrollBar())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Caption")
  {
    if(target->parent() == 0)
    {
      if(target->caption() != 0)
        return 1;
      else
        return 0;
    }
    else
      return 0;
  }
  else if(propStr == "Checked")
  {
    if(tclass == "QCheckBox")
    {
      QCheckBox *tmp = (QCheckBox *)target;
      QCheckBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->isChecked() == cmp.isChecked())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QRadioButton")
    {
      QRadioButton *tmp = (QRadioButton *)target;
      QWidget dummy;
      dummy.setStyle(prefs->defaultStyle());
      QRadioButton cmp(&dummy);
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->isChecked() == cmp.isChecked())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Default")
  {
    if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->isDefault() == cmp.isDefault())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "DragSelect")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->dragSelect() == cmp.dragSelect())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "EchoMode")
  {
    if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      QLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->echoMode() == cmp.echoMode())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Enable")
  {
    if(target->isEnabled())
      return 0;
    else
      return 1;
  }
  else if(propStr == "Exclusive")
  {
    if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->isExclusive() == cmp.isExclusive())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "FocusPolicy")
  {
    if(tclass == "QWidget")
    {
      QWidget cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(target->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QDialog")
    {
      QDialog cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(target->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QCheckBox")
    {
      QCheckBox *tmp = (QCheckBox *)target;
      QCheckBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QRadioButton")
    {
      QRadioButton *tmp = (QRadioButton *)target;
      QWidget dummy;
      dummy.setStyle(prefs->defaultStyle());
      QRadioButton cmp(&dummy);
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      QComboBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QFrame")
    {
      QFrame *tmp = (QFrame *)target;
      QFrame cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QGroupBox")
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMenuBar")
    {
      QMenuBar *tmp = (QMenuBar *)target;
      QMenuBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      QLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->focusPolicy() == cmp.focusPolicy())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Font")
  {
    if(target->font() == QFont::defaultFont())
      return 0;
    else
      return 1;
  }
  else if(propStr == "FrameStyle")
  {
    if(tclass == "QFrame")
    {
      QFrame *tmp = (QFrame *)target;
      QFrame cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QGroupBox")
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMenuBar")
    {
      QMenuBar *tmp = (QMenuBar *)target;
      QMenuBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frameStyle() == cmp.frameStyle())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "GUIStyle")
  {
    QWidget cmp;
    cmp.setStyle(prefs->defaultStyle());
    if(target->style() == cmp.style())
      return 0;
    else
      return 1;
  }
  else if(propStr == "HasFrame")
  {
    if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      QLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->frame() == cmp.frame())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "ItemList")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      if(tmp->count() == 0)
        return 0;
      else
        return 1;
    }
    else if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      if(tmp->count() == 0)
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "LineWidth")
  {
    if(tclass == "QFrame")
    {
      QFrame *tmp = (QFrame *)target;
      QFrame cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QGroupBox")
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMenuBar")
    {
      QMenuBar *tmp = (QMenuBar *)target;
      QMenuBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineWidth() == cmp.lineWidth())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Margin")
  {
    if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->margin() == cmp.margin())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "MidLineWidth")
  {
    if(tclass == "QFrame")
    {
      QFrame *tmp = (QFrame *)target;
      QFrame cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QGroupBox")
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMenuBar")
    {
      QMenuBar *tmp = (QMenuBar *)target;
      QMenuBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->midLineWidth() == cmp.midLineWidth())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "MouseTracking")
  {
    if(tclass == "QWidget")
    {
      QWidget cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(target->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QDialog")
    {
      QDialog cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(target->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QCheckBox")
    {
      QCheckBox *tmp = (QCheckBox *)target;
      QCheckBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QRadioButton")
    {
      QRadioButton *tmp = (QRadioButton *)target;
      QWidget dummy;
      dummy.setStyle(prefs->defaultStyle());
      QRadioButton cmp(&dummy);
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      QComboBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QFrame")
    {
      QFrame *tmp = (QFrame *)target;
      QFrame cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QGroupBox")
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMenuBar")
    {
      QMenuBar *tmp = (QMenuBar *)target;
      QMenuBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      QLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->hasMouseTracking() == cmp.hasMouseTracking())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Orientation")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->orientation() == cmp.orientation())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->orientation() == cmp.orientation())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "OverwriteMode")
  {
    if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->isOverwriteMode() == cmp.isOverwriteMode())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Palette")
  {
    if(tclass == "QWidget" || tclass == "QDialog")
    {
      QWidget cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(target->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QCheckBox")
    {
      QCheckBox *tmp = (QCheckBox *)target;
      QCheckBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      QPushButton cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QRadioButton")
    {
      QRadioButton *tmp = (QRadioButton *)target;
      QWidget dummy;
      dummy.setStyle(prefs->defaultStyle());
      QRadioButton cmp(&dummy);
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      QComboBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QFrame")
    {
      QFrame *tmp = (QFrame *)target;
      QFrame cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QGroupBox")
    {
      QGroupBox *tmp = (QGroupBox *)target;
      QGroupBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QButtonGroup")
    {
      QButtonGroup *tmp = (QButtonGroup *)target;
      QButtonGroup cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      QLabel cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMenuBar")
    {
      QMenuBar *tmp = (QMenuBar *)target;
      QMenuBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      QLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->palette() == cmp.palette())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Range")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->minValue()==cmp.minValue() && tmp->maxValue()==cmp.maxValue() )
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->minValue()==cmp.minValue() && tmp->maxValue()==cmp.maxValue() )
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "ReadOnly")
  {
    if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      QMultiLineEdit cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->isReadOnly() == cmp.isReadOnly())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "SizeLimit")
  {
    if(tclass == "QComboBox")
    {
      QComboBox *tmp = (QComboBox *)target;
      QComboBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->sizeLimit() == cmp.sizeLimit())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "SmoothScroll")
  {
    if(tclass == "QListBox")
    {
      QListBox *tmp = (QListBox *)target;
      QListBox cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->smoothScrolling() == cmp.smoothScrolling())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Steps")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineStep()==cmp.lineStep() && tmp->pageStep()==cmp.pageStep() )
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->lineStep()==cmp.lineStep() && tmp->pageStep()==cmp.pageStep() )
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Text")
  {
    if(target->inherits("QButton"))
    {
      QButton *tmp = (QButton *)target;
      const char *txt = tmp->text();
      if(txt == 0)
        return 0;
      if(*txt == '\0')
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLabel")
    {
      QLabel *tmp = (QLabel *)target;
      const char *txt = tmp->text();
      if(txt == 0)
        return 0;
      if(*txt == '\0')
        return 0;
      else
        return 1;
    }
    else if(tclass == "QLineEdit")
    {
      QLineEdit *tmp = (QLineEdit *)target;
      const char *txt = tmp->text();
      if(txt == 0)
        return 0;
      if(*txt == '\0')
        return 0;
      else
        return 1;
    }
    else if(tclass == "QMultiLineEdit")
    {
      QMultiLineEdit *tmp = (QMultiLineEdit *)target;
      const char *txt = tmp->text();
      if(txt == 0)
        return 0;
      if(*txt == '\0')
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "TickInterval")
  {
    if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->tickInterval() == cmp.tickInterval())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "TickMarks")
  {
    if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->tickmarks() == cmp.tickmarks())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Title")
  {
    if(target->inherits("QGroupBox"))
    {
      QGroupBox *tmp = (QGroupBox *)target;
      const char *txt = tmp->title();
      if(txt == 0)
        return 0;
      if(*txt == '\0')
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "ToggleButton")
  {
    if(tclass == "QPushButton")
    {
      QPushButton *tmp = (QPushButton *)target;
      if(tmp->isToggleButton())
        return 1;
      else
        return 0;
    }
  }
  else if(propStr == "Tracking")
  {
    if(tclass == "QScrollBar")
    {
      QScrollBar *tmp = (QScrollBar *)target;
      QScrollBar cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->tracking() == cmp.tracking())
        return 0;
      else
        return 1;
    }
    else if(tclass == "QSlider")
    {
      QSlider *tmp = (QSlider *)target;
      QSlider cmp;
      cmp.setStyle(prefs->defaultStyle());
      if(tmp->tracking() == cmp.tracking())
        return 0;
      else
        return 1;
    }
  }
  else if(propStr == "Value")
  {
    if(tclass == "QScrollBar" || tclass == "QSlider" )
    {
      QRangeControl *tmp = (QRangeControl *)target;
      if(tmp->value() == 0)
        return 0;
      else
        return 1;
    }
  }
  else
  {
    cerr << "*** Unknown property, cannot check\n";
    return 0;
  }
  cerr << "*** "<< propStr << ": Unknown property for type: "
       << target->className() << endl;
  return 0;
}
