#include "winfomodal.h"

WidgetInfoModal::WidgetInfoModal(QWidget *parent, const char *name)
: QDialog(parent, name, TRUE)
{
  QFont fnt;
  QColor fg, bg, light, dark, mid, text, base;
  QColorGroup normal, disabled, active;
  QPalette pal;

  this->resize(505, 230);
  this->setCaption("New Design");
  nameGrp = new QGroupBox(this);
  nameGrp->setGeometry(0, 0, 185, 50);
  nameGrp->setTitle("Class Name");
  nameEdit = new QLineEdit(nameGrp);
  nameEdit->setGeometry(5, 15, 175, 25);
  argGrp = new QGroupBox(this);
  argGrp->setGeometry(190, 0, 315, 50);
  argGrp->setTitle("Arguments");
  argEdit = new QLineEdit(argGrp);
  argEdit->setGeometry(5, 15, 300, 25);
  argEdit->setText("QWidget *parent, const char *name");
  baseGrp = new QGroupBox(this);
  baseGrp->setGeometry(0, 50, 505, 140);
  baseGrp->setTitle("Base Class");
  btypeCombo = new QComboBox(baseGrp);
  btypeCombo->setGeometry(95, 15, 125, 25);
  btypeCombo->setFocusPolicy((QWidget::FocusPolicy)3);
  btypeCombo->insertItem("QWidget");
  btypeCombo->insertItem("QDialog");
  btypeCombo->insertItem("User Defined");
  modalBox = new QCheckBox(baseGrp);
  modalBox->setGeometry(235, 15, 80, 25);
  modalBox->setEnabled((bool)0);
  modalBox->setText("Modal");
  untitled_0 = new QLabel(baseGrp);
  untitled_0->setGeometry(10, 45, 80, 25);
  untitled_0->setText("Initializer");
  untitled_0->setAlignment(33);
  binitEdit = new QLineEdit(baseGrp);
  binitEdit->setGeometry(95, 45, 340, 25);
  binitEdit->setEnabled((bool)0);
  binitEdit->setText(": QWidget(parent name)");
  untitled_1 = new QLabel(baseGrp);
  untitled_1->setGeometry(10, 75, 80, 25);
  untitled_1->setText("Class");
  untitled_1->setAlignment(33);
  btypeEdit = new QLineEdit(baseGrp);
  btypeEdit->setGeometry(95, 75, 175, 25);
  btypeEdit->setEnabled((bool)0);
  btypeEdit->setText("QWidget");
  untitled_2 = new QLabel(baseGrp);
  untitled_2->setGeometry(270, 75, 80, 25);
  untitled_2->setText("Substitute");
  untitled_2->setAlignment(34);
  subsCombo = new QComboBox(baseGrp);
  subsCombo->setGeometry(355, 75, 80, 25);
  subsCombo->setFocusPolicy((QWidget::FocusPolicy)3);
  subsCombo->setEnabled((bool)0);
  subsCombo->insertItem("QWidget");
  subsCombo->insertItem("QDialog");
  untitled_3 = new QLabel(baseGrp);
  untitled_3->setGeometry(10, 105, 80, 25);
  untitled_3->setText("Header file");
  untitled_3->setAlignment(33);
  inclEdit = new QLineEdit(baseGrp);
  inclEdit->setGeometry(95, 105, 175, 25);
  inclEdit->setEnabled((bool)0);
  inclEdit->setText("qwidget.h");
  okBut = new QPushButton(this);
  okBut->setGeometry(235, 200, 80, 25);
  okBut->setText("&Ok");
  okBut->setDefault((bool)1);
  cancelBut = new QPushButton(this);
  cancelBut->setGeometry(325, 200, 80, 25);
  cancelBut->setText("&Cancel");
  helpBut = new QPushButton(this);
  helpBut->setGeometry(415, 200, 80, 25);
  helpBut->setText("&Help");

  connect(okBut, SIGNAL(clicked()), SLOT(accept()) );
  connect(cancelBut, SIGNAL(clicked()), SLOT(reject()) );
  connect(modalBox, SIGNAL(clicked()), SLOT(toggleModal()) );
  connect(helpBut, SIGNAL(clicked()), SLOT(showHelp()) );
  connect(btypeCombo, SIGNAL(activated(int)), SLOT(newClass(int)) );
  connect(subsCombo, SIGNAL(activated(int)), SLOT(newSubs(int)) );
}

WidgetInfoModal::~WidgetInfoModal()
{
}

WidgetInfo * WidgetInfoModal::getInfo()
{
  WidgetInfo *ret = new WidgetInfo;
  ret->name = nameEdit->text();
  ret->arguments = argEdit->text();
  ret->baseInitializer = binitEdit->text();

  int idx = btypeCombo->currentItem();
  if(idx == 0)
  {
    ret->baseClass = "QWidget";
    ret->baseHeader = "qwidget.h";
    ret->baseSubstitute = "QWidget";
    ret->modal = FALSE;
  }
  else if(idx == 1)
  {
    ret->baseClass = "QDialog";
    ret->baseHeader = "qdialog.h";
    ret->baseSubstitute = "QDialog";
    ret->modal = modalBox->isChecked();
  }
  else //if(idx ==2)
  {
    ret->baseClass = btypeEdit->text();
    ret->baseHeader = inclEdit->text();
    ret->baseSubstitute = subsCombo->currentText();
    ret->modal = modalBox->isChecked();
  }
  ret->baseInitializer = binitEdit->text();
  return ret;
}

void WidgetInfoModal::setInfo(WidgetInfo *info)
{
  nameEdit->setText(info->name);
  argEdit->setText(info->arguments);

  if(info->baseClass == "QWidget")
  {
    btypeCombo->setCurrentItem(0);
    binitEdit->setEnabled(FALSE);
    modalBox->setEnabled(FALSE);
    btypeEdit->setEnabled(FALSE);
    subsCombo->setEnabled(FALSE);
    inclEdit->setEnabled(FALSE);
  }
  else if(info->baseClass == "QDialog")
  {
    btypeCombo->setCurrentItem(1);
    binitEdit->setEnabled(FALSE);
    modalBox->setEnabled(TRUE);
    btypeEdit->setEnabled(FALSE);
    subsCombo->setEnabled(FALSE);
    inclEdit->setEnabled(FALSE);
  }
  else
  {
    btypeCombo->setCurrentItem(2);
    btypeEdit->setEnabled(TRUE);
    binitEdit->setEnabled(TRUE);
    if(info->baseSubstitute == "QWidget")
      modalBox->setEnabled(TRUE);
    else
      modalBox->setEnabled(FALSE);
    inclEdit->setEnabled(TRUE);
    subsCombo->setEnabled(TRUE);
  }

  modalBox->setChecked(info->modal);
  binitEdit->setText(info->baseInitializer);
  btypeEdit->setText(info->baseClass);

  if(info->baseSubstitute == "QWidget")
    subsCombo->setCurrentItem(0);
  else
    subsCombo->setCurrentItem(1);

  inclEdit->setText(info->baseHeader);
}

void WidgetInfoModal::newClass(int idx)
{
  if(idx == 0)
  {
    modalBox->setEnabled(FALSE);
    modalBox->setChecked(FALSE);
    btypeEdit->setEnabled(FALSE);
    subsCombo->setEnabled(FALSE);
    subsCombo->setCurrentItem(0);
    binitEdit->setEnabled(FALSE);
    binitEdit->setText(": QWidget(parent, name)");
    inclEdit->setEnabled(FALSE);
    inclEdit->setText("qwidget.h");
  }
  else if(idx == 1)
  {
    modalBox->setEnabled(TRUE);
    btypeEdit->setEnabled(FALSE);
    btypeEdit->setText("QDialog");
    subsCombo->setEnabled(FALSE);
    subsCombo->setCurrentItem(1);
    binitEdit->setEnabled(FALSE);
    if(modalBox->isChecked())
      binitEdit->setText(": QDialog(parent, name, TRUE)");
    else
      binitEdit->setText(": QDialog(parent, name, FALSE)");
    inclEdit->setEnabled(FALSE);
    inclEdit->setText("qdialog.h");
  }
  else //if(idx == 2)
  {
    modalBox->setEnabled(FALSE);
    btypeEdit->setEnabled(TRUE);
    btypeEdit->setText("QWidget");
    subsCombo->setEnabled(TRUE);
    subsCombo->setCurrentItem(0);
    binitEdit->setEnabled(TRUE);
    binitEdit->setText(": QWidget(parent, name)");
    inclEdit->setEnabled(TRUE);
    inclEdit->setText("qwidget.h");
  }
}

void WidgetInfoModal::showHelp()
{
  QMessageBox::information(this, "Ebuilder",
  "In this dialog, you can specify what the new class you want to design\n"
  "looks like.\n\n"
  "Whatever class you create, it can inherit either QWidget or QDialog.\n"
  "Select the appropriate base class. If you select QDialog as the base\n"
  "class, you can use the check box to make it modal.\n\n"
  "It is possible to use another class as the baseclass for your widget.\n"
  "The other class MUST inherit either QWidget or QDialog. If it doesn't\n"
  "the resulting code will not compile. Consider yourself warned!\n"
  "To have a user define baseclass, select 'User Defined' from the base\n"
  "class combo box. This will enable the rest of the of the base class\n"
  "panel.\n\n"
  "Enter the name of the class you wish to use as the base class. Now\n"
  "you need to choose a substitute class that will be used to mimic your\n"
  "user defined class in the design. Ebuilder is not able to dynamically\n"
  "load your class. You can choose QWidget or QDialog as substitute class.\n"
  "Choose QWidget if your class behaves like a normal widget and QDialog\n"
  "if it behaves like (inherits) QDialog.\n\n"
  "Next, you should enter the initializer that needs to be used in the\n"
  "constructor of the class you are designing. Use the same names for the\n"
  "arguments as you specified on the top of the page.\n\n"
  "Finally, specify the name of the header file that includes the\n"
  "declaration of the base class you are using.");
}

void WidgetInfoModal::newSubs(int idx)
{
  if(idx == 0)
    modalBox->setEnabled(FALSE);
  else
    modalBox->setEnabled(TRUE);
}

void WidgetInfoModal::toggleModal()
{
  if(btypeCombo->currentItem() == 2)
    return;
  if(modalBox->isChecked())
    binitEdit->setText(": QDialog(parent, name, TRUE)");
  else
    binitEdit->setText(": QDialog(parent, name, FALSE)");
}
